@@ -1,12 +1,12 @@
 /***************************************************************************
- * CVSID: $Id: device.c,v 1.16 2005/02/28 04:53:15 david Exp $
+ * CVSID: $Id: device.c,v 1.21 2006/01/21 06:36:51 david Exp $
  *
  * device.c : HalDevice methods
  *
  * Copyright (C) 2003 David Zeuthen, <david@fubar.dk>
  * Copyright (C) 2004 Novell, Inc.
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  **************************************************************************/
 
@@ -147,7 +147,8 @@ hal_device_get_type (void)
 			NULL, NULL,
 			sizeof (HalDevice),
 			0,
-			(GInstanceInitFunc) hal_device_init
+			(GInstanceInitFunc) hal_device_init,
+			NULL
 		};
 
 		type = g_type_register_static (G_TYPE_OBJECT,
@@ -165,7 +166,7 @@ hal_device_new (void)
 {
 	HalDevice *device;
 
-	device = g_object_new (HAL_TYPE_DEVICE, NULL);
+	device = g_object_new (HAL_TYPE_DEVICE, NULL, NULL);
 
 #ifdef HALD_MEMLEAK_DBG
 	dbg_hal_device_object_delta++;
@@ -1090,6 +1091,25 @@ hal_device_property_get_strlist (HalDevice    *device,
 		return NULL;
 }
 
+const char *
+hal_device_property_get_strlist_elem (HalDevice    *device,
+				      const char   *key,
+				      guint index)
+{
+	GSList *strlist;
+	GSList *i;
+
+	strlist = hal_device_property_get_strlist (device, key);
+	if (strlist == NULL)
+		return NULL;
+
+	i = g_slist_nth (strlist, index);
+	if (i == NULL)
+		return NULL;
+
+	return (const char *) i->data;
+}
+
 gboolean
 hal_device_property_strlist_append (HalDevice    *device,
 				    const char   *key,
@@ -1179,6 +1199,39 @@ hal_device_property_strlist_remove_elem (HalDevice    *device,
 	return FALSE;
 }
 
+gboolean
+hal_device_property_strlist_clear (HalDevice    *device,
+				   const char   *key)
+{
+	HalProperty *prop;
+
+	/* check if property already exists */
+	prop = hal_device_property_find (device, key);
+
+	if (prop == NULL) {
+		prop = hal_property_new_strlist (key);
+
+		device->properties = g_slist_prepend (device->properties, prop);
+
+		g_signal_emit (device, signals[PROPERTY_CHANGED], 0,
+			       key, FALSE, TRUE);
+
+		return TRUE;
+	}
+
+	if (hal_property_get_type (prop) != HAL_PROPERTY_TYPE_STRLIST)
+		return FALSE;
+	
+	if (hal_property_strlist_clear (prop)) {
+		g_signal_emit (device, signals[PROPERTY_CHANGED], 0,
+			       key, FALSE, FALSE);
+		return TRUE;
+	}
+	
+	return FALSE;
+}
+
+
 gboolean
 hal_device_property_strlist_add (HalDevice *device,
 				 const char *key,
