@@ -1,11 +1,11 @@
 /***************************************************************************
- * CVSID: $Id: device_info.c,v 1.27 2005/05/02 18:48:29 david Exp $
+ * CVSID: $Id: device_info.c,v 1.32 2006/01/21 06:36:51 david Exp $
  *
  * device_store.c : Search for .fdi files and merge on match
  *
  * Copyright (C) 2003 David Zeuthen, <david@fubar.dk>
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  **************************************************************************/
 
@@ -79,24 +79,28 @@ enum {
 	/** Processing an append element */
 	CURELEM_APPEND = 4,
 
-	/** Processing an prepend element */
+	/** Processing a prepend element */
 	CURELEM_PREPEND = 5,
 
-	/** Processing an prepend element */
-	CURELEM_REMOVE = 6
+	/** Processing a remove element */
+	CURELEM_REMOVE = 6,
+
+	/** Processing a clear element */
+	CURELEM_CLEAR = 7
 };
 
 /** What and how to merge */
 enum {
-	MERGE_TYPE_UNKNOWN,
-	MERGE_TYPE_STRING,
-	MERGE_TYPE_BOOLEAN,
-	MERGE_TYPE_INT32,
-	MERGE_TYPE_UINT64,
-	MERGE_TYPE_DOUBLE,
-	MERGE_TYPE_COPY_PROPERTY,
-	MERGE_TYPE_STRLIST,
-	MERGE_TYPE_REMOVE
+	MERGE_TYPE_UNKNOWN       = 0,
+	MERGE_TYPE_STRING        = 1,
+	MERGE_TYPE_BOOLEAN       = 2,
+	MERGE_TYPE_INT32         = 3,
+	MERGE_TYPE_UINT64        = 4,
+	MERGE_TYPE_DOUBLE        = 5,
+	MERGE_TYPE_COPY_PROPERTY = 6,
+	MERGE_TYPE_STRLIST       = 7,
+	MERGE_TYPE_REMOVE        = 8,
+	MERGE_TYPE_CLEAR         = 9
 };
 
 /** Parsing Context
@@ -190,7 +194,6 @@ resolve_udiprop_path (const char *path, const char *source_udi,
 		goto out;
 	}
 
-	
 	/* Start with the source udi */
 	strncpy (udi_result, source_udi, udi_result_size);
 
@@ -232,7 +235,7 @@ resolve_udiprop_path (const char *path, const char *source_udi,
 			strncpy (udi_result, newudi, udi_result_size);
 		} else {
 			/*HAL_INFO (("new_udi = '%s'", curtoken));*/
-			strncpy (udi_result, curtoken, udi_result_size);			
+			strncpy (udi_result, curtoken, udi_result_size);
 		}
 
 	}
@@ -528,7 +531,6 @@ handle_match (ParsingContext * pc, const char **attr)
 
 		/*HAL_INFO (("is_absolute=%d, should_be=%d, path='%s'", is_absolute_path, should_be_absolute_path, path));*/
 
-
 		if (should_be_absolute_path) {
 			if (is_absolute_path)
 				return TRUE;
@@ -746,7 +748,7 @@ handle_append_prepend (ParsingContext * pc, const char **attr)
 	return;
 }
 
-/** Called when the append or prepend element begins.
+/** Called when the remove element begins.
  *
  *  @param  pc                  Parsing context
  *  @param  attr                Attribute key/value pairs
@@ -788,6 +790,39 @@ handle_remove (ParsingContext * pc, const char **attr)
 	return;
 }
 
+/** Called when the clear element begins.
+ *
+ *  @param  pc                  Parsing context
+ *  @param  attr                Attribute key/value pairs
+ */
+static void
+handle_clear (ParsingContext * pc, const char **attr)
+{
+	int num_attrib;
+
+	pc->merge_type = MERGE_TYPE_UNKNOWN;
+
+	for (num_attrib = 0; attr[num_attrib] != NULL; num_attrib++) {
+		;
+	}
+
+	if (num_attrib != 4)
+		return;
+	
+	if (strcmp (attr[0], "key") != 0)
+		return;
+
+
+	if (strcmp (attr[3], "strlist") != 0)
+		return;
+	
+	strncpy (pc->merge_key, attr[1], MAX_KEY_SIZE);
+	
+	pc->merge_type = MERGE_TYPE_CLEAR;
+
+	return;
+}
+
 /** Abort parsing of document
  *
  *  @param  pc                  Parsing context
@@ -916,6 +951,23 @@ start (ParsingContext * pc, const char *el, const char **attr)
 		} else {
 			/*HAL_INFO(("No merge!")); */
 		}
+	} else if (strcmp (el, "clear") == 0) {
+		if (pc->curelem != CURELEM_DEVICE
+		    && pc->curelem != CURELEM_MATCH) {
+			HAL_ERROR (("%s:%d:%d: Element <remove> can only be "
+				    "inside <device> and <match>", 
+				    pc->file, 
+				    XML_GetCurrentLineNumber (pc->parser), 
+				    XML_GetCurrentColumnNumber (pc->parser)));
+			parsing_abort (pc);
+		}
+
+		pc->curelem = CURELEM_CLEAR;
+		if (pc->match_ok) {
+			handle_clear (pc, attr);
+		} else {
+			/*HAL_INFO(("No merge!")); */
+		}
 	} else if (strcmp (el, "device") == 0) {
 		if (pc->curelem != CURELEM_DEVICE_INFO) {
 			HAL_ERROR (("%s:%d:%d: Element <device> can only be "
@@ -1149,6 +1201,10 @@ end (ParsingContext * pc, const char *el)
 				hal_device_property_remove (pc->device, pc->merge_key);
 			}
 		}
+	} else if (pc->curelem == CURELEM_CLEAR && pc->match_ok) {
+		if (pc->merge_type == MERGE_TYPE_CLEAR) {
+			hal_device_property_strlist_clear (pc->device, pc->merge_key);
+		}
 	}
 
 
@@ -1213,6 +1269,7 @@ process_fdi_file (const char *dir, const char *filename,
 	char buf[512];
 	FILE *file;
 	int filesize;
+	size_t read;
 	char *filebuf;
 	dbus_bool_t device_matched;
 	XML_Parser parser;
@@ -1244,7 +1301,7 @@ process_fdi_file (const char *dir, const char *filename,
 		HAL_ERROR (("Could not allocate %d bytes for file %s", filesize, buf));
 		goto out;
 	}
-	fread (filebuf, sizeof (char), filesize, file);
+	read = fread (filebuf, sizeof (char), filesize, file);
 
 	/* initialize parsing context */
 	parsing_context =
