@@ -1,12 +1,12 @@
 /***************************************************************************
- * CVSID: $Id: device_store.c,v 1.22 2005/01/31 20:06:39 david Exp $
+ * CVSID: $Id: device_store.c,v 1.26 2005/11/02 15:38:13 david Exp $
  *
  * device_store.c : HalDeviceStore methods
  *
  * Copyright (C) 2003 David Zeuthen, <david@fubar.dk>
  * Copyright (C) 2004 Novell, Inc.
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  **************************************************************************/
 
@@ -140,7 +140,7 @@ hal_device_store_new (void)
 {
 	HalDeviceStore *store;
 
-	store = g_object_new (HAL_TYPE_DEVICE_STORE, NULL);
+	store = g_object_new (HAL_TYPE_DEVICE_STORE, NULL, NULL);
 
 	return store;
 }
@@ -172,6 +172,14 @@ emit_device_capability_added (HalDevice *device,
 void
 hal_device_store_add (HalDeviceStore *store, HalDevice *device)
 {
+	const char buf[] = "/org/freedesktop/Hal/devices/";
+
+	if (strncmp(device->udi, buf, sizeof (buf) - 1) != 0) {
+		
+		HAL_ERROR(("Can't add HalDevice with incorrect UDI. Valid "
+			   "UDI must start with '/org/freedesktop/Hal/devices/'"));
+		goto out;
+	}
 	store->devices = g_slist_prepend (store->devices,
 					  g_object_ref (device));
 
@@ -181,6 +189,9 @@ hal_device_store_add (HalDeviceStore *store, HalDevice *device)
 			  G_CALLBACK (emit_device_capability_added), store);
 
 	g_signal_emit (store, signals[STORE_CHANGED], 0, device, TRUE);
+
+out:
+	;
 }
 
 gboolean
