@@ -1,11 +1,11 @@
 /***************************************************************************
- * CVSID: $Id: hald_dbus.c,v 1.38 2005/07/08 23:02:29 david Exp $
+ * CVSID: $Id: hald_dbus.c,v 1.57 2006/02/20 02:24:27 david Exp $
  *
  * dbus.c : D-BUS interface of HAL daemon
  *
  * Copyright (C) 2003 David Zeuthen, <david@fubar.dk>
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  **************************************************************************/
 
@@ -32,7 +32,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <stdarg.h>
-#include <sys/types.h>
+#include <stdint.h>
 #include <sys/time.h>
 
 #include <dbus/dbus.h>
@@ -46,6 +46,7 @@
 #include "logger.h"
 #include "osspec.h"
 #include "util.h"
+#include "hald_runner.h"
 
 static DBusConnection *dbus_connection = NULL;
 
@@ -1393,6 +1394,57 @@ device_string_list_append_prepend (DBusConnection * connection, DBusMessage * me
 	return DBUS_HANDLER_RESULT_HANDLED;
 }
 
+/* TODO: docs */
+static DBusHandlerResult
+device_string_list_remove (DBusConnection * connection, DBusMessage * message)
+{
+	const char *udi;
+	const char *key;
+	const char *value;
+	HalDevice *d;
+	DBusMessage *reply;
+	DBusError error;
+	gboolean ret;
+
+	HAL_TRACE (("entering"));
+
+	udi = dbus_message_get_path (message);
+
+	d = hal_device_store_find (hald_get_gdl (), udi);
+	if (d == NULL)
+		d = hal_device_store_find (hald_get_tdl (), udi);
+
+	if (d == NULL) {
+		raise_no_such_device (connection, message, udi);
+		return DBUS_HANDLER_RESULT_HANDLED;
+	}
+
+	dbus_error_init (&error);
+	if (!dbus_message_get_args (message, &error,
+				    DBUS_TYPE_STRING, &key,
+				    DBUS_TYPE_STRING, &value,
+				    DBUS_TYPE_INVALID)) {
+		raise_syntax (connection, message, "StringListRemove");
+		return DBUS_HANDLER_RESULT_HANDLED;
+	}
+
+	ret = hal_device_property_strlist_remove (d, key, value);
+	if (!ret) {
+		raise_property_type_error (connection, message, udi, key);
+		return DBUS_HANDLER_RESULT_HANDLED;
+	}
+
+	reply = dbus_message_new_method_return (message);
+	if (reply == NULL)
+		DIE (("No memory"));
+	
+	if (!dbus_connection_send (connection, reply, NULL))
+		DIE (("No memory"));
+	
+	dbus_message_unref (reply);
+	return DBUS_HANDLER_RESULT_HANDLED;
+}
+
 
 
 /** Remove a property on a device.
@@ -1540,7 +1592,7 @@ device_query_capability (DBusConnection * connection,
 {
 	dbus_bool_t rc;
 	const char *udi;
-	const char *caps;
+	GSList *caps;
 	char *capability;
 	HalDevice *d;
 	DBusMessage *reply;
@@ -1573,23 +1625,18 @@ device_query_capability (DBusConnection * connection,
 		DIE (("No memory"));
 
 	rc = FALSE;
-	caps = hal_device_property_get_string (d, "info.capabilities");
+	caps = hal_device_property_get_strlist (d, "info.capabilities");
 	if (caps != NULL) {
-		char **capsv, **iter;
+		GSList *iter;
 
-		capsv = g_strsplit (caps, " ", 0);
-		for (iter = capsv; *iter != NULL; iter++) {
-			if (strcmp (*iter, capability) == 0) {
+		for (iter = caps; iter != NULL; iter=g_slist_next(iter)) {
+			if (strcmp (iter->data, capability) == 0) {
 				rc = TRUE;
 				break;
 			}
 		}
-
-		g_strfreev (capsv);
 	}
 
-	dbus_free (capability);
-
 	dbus_message_iter_init_append (reply, &iter);
 	dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &rc);
 
@@ -2481,6 +2528,353 @@ manager_commit_to_gdl (DBusConnection * connection, DBusMessage * message, dbus_
 	return DBUS_HANDLER_RESULT_HANDLED;
 }
 
+typedef struct {
+	char *udi;
+	char *execpath;
+	char **extra_env;
+	char *stdin;
+	DBusMessage *message;
+	DBusConnection *connection;
+} MethodInvocation;
+
+static void
+hald_exec_method_cb (HalDevice *d, guint32 exit_type, 
+		     gint return_code, gchar **error,
+		     gpointer data1, gpointer data2);
+
+
+static void
+hald_exec_method_free_mi (MethodInvocation *mi)
+{
+	g_free (mi->udi);
+	g_free (mi->execpath);
+	g_strfreev (mi->extra_env);
+	g_free (mi->stdin);
+	g_free (mi);
+}
+
+/* returns FALSE if we don't actually invoke anything */
+static gboolean
+hald_exec_method_do_invocation (MethodInvocation *mi)
+{
+	gboolean ret;
+	HalDevice *d;
+
+	ret = FALSE;
+
+	d = hal_device_store_find (hald_get_gdl (), mi->udi);
+	if (d == NULL)
+		d = hal_device_store_find (hald_get_tdl (), mi->udi);
+
+	if (d != NULL) {
+		/* no timeout */
+		hald_runner_run_method(d, 
+				       mi->execpath, 
+				       mi->extra_env, 
+				       mi->stdin, 
+				       TRUE,
+				       0,
+				       hald_exec_method_cb,
+				       (gpointer) mi->message, 
+				       (gpointer) mi->connection);
+
+		ret = TRUE;
+	} else {
+		HAL_WARNING (("In-queue method call on non-existant device"));
+
+		raise_no_such_device (mi->connection, mi->message, mi->udi);
+	}
+
+	return ret;
+}
+
+
+static GHashTable *udi_to_method_queue = NULL;
+
+static void 
+hald_exec_method_process_queue (const char *udi);
+
+static void
+hald_exec_method_enqueue (MethodInvocation *mi)
+{
+	gpointer origkey;
+	GList *queue;
+
+	if (udi_to_method_queue == NULL) {
+		udi_to_method_queue = g_hash_table_new (g_str_hash,
+							g_str_equal);
+	}
+
+	if (g_hash_table_lookup_extended (udi_to_method_queue, mi->udi, &origkey, (gpointer) &queue)) {
+		HAL_INFO (("enqueue"));;
+		queue = g_list_append (queue, mi);
+		g_hash_table_replace (udi_to_method_queue, g_strdup (mi->udi), queue);
+	} else {
+		HAL_INFO (("no need to enqueue"));;
+		queue = g_list_append (NULL, mi);
+		g_hash_table_insert (udi_to_method_queue, g_strdup (mi->udi), queue);
+
+		hald_exec_method_do_invocation (mi);
+		hald_exec_method_free_mi (mi);
+	}
+}
+
+
+static void 
+hald_exec_method_process_queue (const char *udi)
+{
+	gpointer origkey;
+	GList *queue;
+
+	if (g_hash_table_lookup_extended (udi_to_method_queue, udi, &origkey, (gpointer) &queue)) {
+		if (queue != NULL) {
+			queue = g_list_delete_link (queue, queue);
+		}
+
+		if (queue == NULL) {
+			HAL_INFO (("No more methods in queue"));;
+			g_hash_table_remove (udi_to_method_queue, udi);
+		} else {
+			MethodInvocation *mi;
+
+			HAL_INFO (("Execing next method in queue"));;
+			g_hash_table_replace (udi_to_method_queue, g_strdup (udi), queue);
+
+			mi = (MethodInvocation *) queue->data;
+
+			if (!hald_exec_method_do_invocation (mi)) {
+				/* the device went away before we got to it... */
+				hald_exec_method_process_queue (mi->udi);
+			}
+
+			hald_exec_method_free_mi (mi);
+
+		}
+	}
+}
+
+static void
+hald_exec_method_cb (HalDevice *d, guint32 exit_type, 
+		     gint return_code, gchar **error,
+		     gpointer data1, gpointer data2)
+{
+	dbus_uint32_t result;
+	DBusMessage *reply = NULL;
+	DBusMessage *message;
+	DBusMessageIter iter;
+	DBusConnection *conn;
+	gchar *exp_name = NULL;
+	gchar *exp_detail = NULL;
+
+	hald_exec_method_process_queue (d->udi);
+
+	message = (DBusMessage *) data1;
+	conn = (DBusConnection *) data2;
+	
+	if (exit_type == HALD_RUN_SUCCESS && error != NULL && 
+	    error[0] != NULL && error[1] != NULL) {
+		exp_name = error[0];
+		if (error[0] != NULL) {
+			exp_detail = error[1];
+		}
+		HAL_INFO (("failed with '%s' '%s'", exp_name, exp_detail));
+	}
+	
+	if (exit_type != HALD_RUN_SUCCESS) {
+		reply = dbus_message_new_error (message, "org.freedesktop.Hal.Device.UnknownError", "An unknown error occured");
+		if (conn != NULL) {
+			if (!dbus_connection_send (conn, reply, NULL))
+				DIE (("No memory"));
+		}
+		dbus_message_unref (reply);
+	} else if (exp_name != NULL && exp_detail != NULL) {
+		reply = dbus_message_new_error (message, exp_name, exp_detail);
+		if (reply == NULL) {
+			/* error name may be invalid - assume caller fucked up and use a generic HAL error name */
+			reply = dbus_message_new_error (message, "org.freedesktop.Hal.Device.UnknownError", "An unknown error occured");
+			if (reply == NULL) {
+				DIE (("No memory"));
+			}
+		}
+		if (conn != NULL) {
+			if (!dbus_connection_send (conn, reply, NULL))
+				DIE (("No memory"));
+		}
+		dbus_message_unref (reply);
+
+	} else {
+		result = (dbus_uint32_t) return_code;
+
+		reply = dbus_message_new_method_return (message);
+		if (reply == NULL)
+			DIE (("No memory"));
+		
+		dbus_message_iter_init_append (reply, &iter);
+		dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &result);
+		
+		if (conn != NULL) {
+			if (!dbus_connection_send (conn, reply, NULL))
+				DIE (("No memory"));
+		}
+
+		dbus_message_unref (reply);
+	}
+
+	dbus_message_unref (message);
+}
+
+static DBusHandlerResult
+hald_exec_method (HalDevice *d, DBusConnection *connection, dbus_bool_t local_interface, 
+		  DBusMessage *message, const char *execpath)
+{
+	int type;
+	GString *stdin_str;
+	DBusMessageIter iter;
+	char *extra_env[2];
+	char uid_export[128];
+	MethodInvocation *mi;
+
+	/* add calling uid */
+	extra_env[0] = NULL;
+	if (local_interface) {
+		extra_env[0] = "HAL_METHOD_INVOKED_BY_UID=0";
+	} else {
+		const char *sender;
+		
+		sender = dbus_message_get_sender (message);
+		if (sender != NULL) {
+			DBusError error;
+			unsigned long uid;
+			
+			dbus_error_init (&error);
+			uid = dbus_bus_get_unix_user (connection, sender, &error);
+			if (!dbus_error_is_set (&error)) {
+				sprintf (uid_export, "HAL_METHOD_INVOKED_BY_UID=%lu", uid);
+				extra_env[0] = uid_export;
+			}
+		}
+	}
+	extra_env[1] = NULL;
+
+	/* prepare stdin with parameters */
+	stdin_str = g_string_sized_new (256); /* default size for passing params; can grow */
+	dbus_message_iter_init (message, &iter);
+	while ((type = dbus_message_iter_get_arg_type (&iter)) != DBUS_TYPE_INVALID) {
+		switch (type) {
+		case DBUS_TYPE_BYTE:
+		{
+			unsigned char value;
+			dbus_message_iter_get_basic (&iter, &value);
+			g_string_append_printf (stdin_str, "%u", value);
+			break;
+		}
+		case DBUS_TYPE_INT16:
+		{
+			dbus_int16_t value;
+			dbus_message_iter_get_basic (&iter, &value);
+			g_string_append_printf (stdin_str, "%d", value);
+			break;
+		}
+		case DBUS_TYPE_UINT16:
+		{
+			dbus_uint16_t value;
+			dbus_message_iter_get_basic (&iter, &value);
+			g_string_append_printf (stdin_str, "%u", value);
+			break;
+		}
+		case DBUS_TYPE_INT32:
+		{
+			dbus_int32_t value;
+			dbus_message_iter_get_basic (&iter, &value);
+			g_string_append_printf (stdin_str, "%d", value);
+			break;
+		}
+		case DBUS_TYPE_UINT32:
+		{
+			dbus_uint32_t value;
+			dbus_message_iter_get_basic (&iter, &value);
+			g_string_append_printf (stdin_str, "%u", value);
+			break;
+		}
+		case DBUS_TYPE_INT64:
+		{
+			dbus_int64_t value;
+			dbus_message_iter_get_basic (&iter, &value);
+			g_string_append_printf (stdin_str, "%lld", value);
+			break;
+		}
+		case DBUS_TYPE_UINT64:
+		{
+			dbus_uint64_t value;
+			dbus_message_iter_get_basic (&iter, &value);
+			g_string_append_printf (stdin_str, "%llu", value);
+			break;
+		}
+		case DBUS_TYPE_DOUBLE:
+		{
+			double value;
+			dbus_message_iter_get_basic (&iter, &value);
+			g_string_append_printf (stdin_str, "%g", value);
+			break;
+		}
+		case DBUS_TYPE_BOOLEAN:
+		{
+			dbus_bool_t value;
+			dbus_message_iter_get_basic (&iter, &value);
+			g_string_append (stdin_str, value ? "true" : "false");
+			break;
+		}
+		case DBUS_TYPE_STRING:
+		{
+			char *value;
+			dbus_message_iter_get_basic (&iter, &value);
+			g_string_append (stdin_str, value);
+			break;
+		}
+
+		case DBUS_TYPE_ARRAY:
+		{
+			DBusMessageIter iter_strlist;
+			if (dbus_message_iter_get_element_type (&iter) != DBUS_TYPE_STRING)
+				goto error;
+
+			dbus_message_iter_recurse (&iter, &iter_strlist);
+			while (dbus_message_iter_get_arg_type (&iter_strlist) == DBUS_TYPE_STRING) {
+				const char *value;
+				dbus_message_iter_get_basic (&iter_strlist, &value);
+				g_string_append (stdin_str, value);
+				g_string_append (stdin_str, "\t");
+				dbus_message_iter_next(&iter_strlist);
+			}
+			break;
+		}
+
+		default:
+			goto error;
+		}
+
+		g_string_append_c (stdin_str, '\n');
+		dbus_message_iter_next (&iter);
+	}
+
+	mi = g_new0 (MethodInvocation, 1);
+	mi->udi = g_strdup (d->udi);
+	mi->execpath = g_strdup (execpath);
+	mi->extra_env = g_strdupv (extra_env);
+	mi->stdin = g_strdup (stdin_str->str);
+	mi->message = message;
+	mi->connection = connection;
+	hald_exec_method_enqueue (mi);
+
+	dbus_message_ref (message);
+	g_string_free (stdin_str, TRUE);
+
+	return DBUS_HANDLER_RESULT_HANDLED;
+
+error:
+	g_string_free (stdin_str, TRUE);
+	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
 
 static DBusHandlerResult
 hald_dbus_filter_handle_methods (DBusConnection *connection, DBusMessage *message, 
@@ -2618,6 +3012,10 @@ hald_dbus_filter_handle_methods (DBusConnection *connection, DBusMessage *messag
 						"org.freedesktop.Hal.Device",
 						"StringListPrepend")) {
 		return device_string_list_append_prepend (connection, message, TRUE);
+	} else if (dbus_message_is_method_call (message,
+						"org.freedesktop.Hal.Device",
+						"StringListRemove")) {
+		return device_string_list_remove (connection, message);
 	} else if (dbus_message_is_method_call (message,
 						"org.freedesktop.Hal.Device",
 						"Rescan")) {
@@ -2630,11 +3028,77 @@ hald_dbus_filter_handle_methods (DBusConnection *connection, DBusMessage *messag
 						"org.freedesktop.Hal.Device",
 						"EmitCondition")) {
 		return device_emit_condition (connection, message, local_interface);
-	} else
-		osspec_filter_function (connection, message, user_data);
+	} else {
+		const char *interface;
+		const char *udi;
+		const char *method;
+		const char *signature;
+		HalDevice *d;
 
-	return DBUS_HANDLER_RESULT_HANDLED;
+		/* check for device-specific interfaces that individual objects may support */
+
+		udi = dbus_message_get_path (message);
+		interface = dbus_message_get_interface (message);
+		method = dbus_message_get_member (message);
+		signature = dbus_message_get_signature (message);
+
+		d = NULL;
+
+		if (method != NULL) {
+			d = hal_device_store_find (hald_get_gdl (), udi);
+			if (d == NULL)
+				d = hal_device_store_find (hald_get_tdl (), udi);
+		}
+
+		if (d != NULL && interface != NULL && method != NULL && signature != NULL) {
+			GSList *interfaces;
+			GSList *i;
+
+			interfaces = hal_device_property_get_strlist (d, "info.interfaces");
+			for (i = interfaces; i != NULL; i = g_slist_next (i)) {
+				const char *ifname = (const char *) i->data;
+
+				if (strcmp (ifname, interface) == 0) {
+					guint num;
+					GSList *method_names;
+					char *s;
+
+					s = g_strdup_printf ("%s.method_names", interface);
+					method_names = hal_device_property_get_strlist (d, s);
+					g_free (s);
+					for (i = method_names, num = 0; i != NULL; i = g_slist_next (i), num++) {
+						const char *methodname = (const char *) i->data;
+						if (strcmp (methodname, method) == 0) {
+							const char *execpath;
+							const char *sig;
+
+							s = g_strdup_printf ("%s.method_execpaths", interface);
+							execpath = hal_device_property_get_strlist_elem (d, s, num);
+							g_free (s);
+							s = g_strdup_printf ("%s.method_signatures", interface);
+							sig = hal_device_property_get_strlist_elem (d, s, num);
+							g_free (s);
+							
+							if (execpath != NULL && sig != NULL && 
+							    strcmp (sig, signature) == 0) {
+
+								HAL_INFO (("OK for method '%s' with signature '%s' on interface '%s' for UDI '%s' and execpath '%s'", method, signature, interface, udi, execpath));
+
+								return hald_exec_method (d, connection, local_interface,
+											 message, execpath);
+							}
+							
+						}
+					}
+				}
+			}
+			
+		}
+	}
+		
+	return osspec_filter_function (connection, message, user_data);
 }
+       
 
 /** Message handler for method invocations. All invocations on any object
  *  or interface is routed through this function.
@@ -2648,12 +3112,10 @@ DBusHandlerResult
 hald_dbus_filter_function (DBusConnection * connection,
 			   DBusMessage * message, void *user_data)
 {
-	/*
-	HAL_INFO (("obj_path=%s interface=%s method=%s", 
+	/*HAL_INFO (("obj_path=%s interface=%s method=%s", 
 		   dbus_message_get_path(message), 
 		   dbus_message_get_interface(message),
-		   dbus_message_get_member(message)));
-	*/
+		   dbus_message_get_member(message)));*/
 
 	if (dbus_message_is_signal (message, DBUS_INTERFACE_LOCAL, "Disconnected") &&
 	    strcmp (dbus_message_get_path (message), DBUS_PATH_LOCAL) == 0) {
@@ -2686,11 +3148,13 @@ local_server_message_handler (DBusConnection *connection,
 			      DBusMessage *message, 
 			      void *user_data)
 {
+/*
 	HAL_INFO (("local_server_message_handler: destination=%s obj_path=%s interface=%s method=%s", 
 		   dbus_message_get_destination (message), 
 		   dbus_message_get_path (message), 
 		   dbus_message_get_interface (message),
 		   dbus_message_get_member (message)));
+*/
 
 	if (dbus_message_is_method_call (message, "org.freedesktop.DBus", "AddMatch")) {
 		DBusMessage *reply;
