@@ -1,12 +1,13 @@
 /***************************************************************************
- * CVSID: $Id: classdev.c,v 1.27 2005/06/27 19:13:03 david Exp $
+ * CVSID: $Id: classdev.c,v 1.42 2006/01/28 22:51:18 david Exp $
  *
  * classdev.c : Handling of functional kernel devices
  *
  * Copyright (C) 2004 David Zeuthen, <david@fubar.dk>
  * Copyright (C) 2005 Richard Hughes, <richard@hughsie.com>
+ * Copyright (C) 2005 Danny Kukawka, <danny.kukawka@web.de>
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  **************************************************************************/
 
@@ -34,7 +35,7 @@
 #include <string.h>
 #include <mntent.h>
 #include <errno.h>
-#include <sys/types.h>
+#include <stdint.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -45,15 +46,13 @@
 
 #include <limits.h>
 #include <errno.h>
-#include <sys/types.h>
+#include <stdint.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
 #include <net/if_arp.h> /* for ARPHRD_... */
 #include <sys/socket.h>
 #include <linux/types.h>
-#include <linux/netlink.h>
-#include <linux/rtnetlink.h>
 #include <net/if.h>
 
 #include <glib.h>
@@ -64,7 +63,9 @@
 #include "../logger.h"
 #include "../hald.h"
 #include "../device_info.h"
+#include "../device_store.h"
 #include "../util.h"
+#include "../hald_runner.h"
 
 #include "osspec_linux.h"
 
@@ -207,8 +208,6 @@ net_add (const gchar *sysfs_path, const gchar *device_file, HalDevice *physdev,
 		goto error;
 	hal_device_property_set_bool (d, "net.interface_up", flags & IFF_UP);
 
-	hal_device_property_set_string (d, "info.product", "Networking Interface");
-
 	media_type = hal_device_property_get_int (d, "net.arp_proto_hw_id");
 	if (media_type == ARPHRD_ETHER) {
 		FILE *f;
@@ -251,9 +250,11 @@ net_add (const gchar *sysfs_path, const gchar *device_file, HalDevice *physdev,
 		snprintf (wireless_path, SYSFS_PATH_MAX, "%s/wireless", sysfs_path);
 		if (stat (wireless_path, &statbuf) == 0) {
 		*/
+			hal_device_property_set_string (d, "info.product", "WLAN Interface");
 			hal_device_property_set_string (d, "info.category", "net.80211");
 			hal_device_add_capability (d, "net.80211");
 		} else {
+			hal_device_property_set_string (d, "info.product", "Networking Interface");
 			hal_device_property_set_string (d, "info.category", "net.80203");
 			hal_device_add_capability (d, "net.80203");
 		}
@@ -313,6 +314,43 @@ net_compute_udi (HalDevice *d)
 
 /*--------------------------------------------------------------------------------------------------------------*/
 
+static HalDevice *
+scsi_generic_add (const gchar *sysfs_path, const gchar *device_file, HalDevice *physdev, const gchar *sysfs_path_in_devices)
+{
+	HalDevice *d;
+
+	d = NULL;
+
+	if (physdev == NULL || sysfs_path_in_devices == NULL)
+		goto out;
+
+	d = hal_device_new ();
+	hal_device_property_set_string (d, "linux.sysfs_path", sysfs_path);
+	hal_device_property_set_string (d, "info.parent", physdev->udi);
+	hal_device_property_set_string (d, "info.category", "scsi_generic");
+	hal_device_add_capability (d, "scsi_generic");
+	hal_device_property_set_string (d, "info.product", "SCSI Generic Interface");
+	hal_device_property_set_string (d, "scsi_generic.device", device_file);
+
+out:
+	return d;
+}
+
+static gboolean
+scsi_generic_compute_udi (HalDevice *d)
+{
+	gchar udi[256];
+
+	hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
+			      "%s_scsi_generic",
+			      hal_device_property_get_string (d, "info.parent"));
+	hal_device_set_udi (d, udi);
+	hal_device_property_set_string (d, "info.udi", udi);
+	return TRUE;
+}
+
+/*--------------------------------------------------------------------------------------------------------------*/
+
 static HalDevice *
 scsi_host_add (const gchar *sysfs_path, const gchar *device_file, HalDevice *physdev, const gchar *sysfs_path_in_devices)
 {
@@ -444,6 +482,120 @@ usbclass_compute_udi (HalDevice *d)
 	return TRUE;
 }
 
+/*--------------------------------------------------------------------------------------------------------------*/
+
+static HalDevice *
+usbraw_add (const gchar *sysfs_path, const gchar *device_file, HalDevice *physdev, const gchar *sysfs_path_in_devices)
+{
+	HalDevice *d;
+
+	d = NULL;
+
+	if (physdev == NULL || sysfs_path_in_devices == NULL)
+		goto out;
+
+	d = hal_device_new ();
+	hal_device_property_set_string (d, "linux.sysfs_path", sysfs_path);
+	hal_device_property_set_string (d, "info.parent", physdev->udi);
+	hal_device_property_set_string (d, "info.category", "usbraw");
+	hal_device_add_capability (d, "usbraw");
+	hal_device_property_set_string (d, "info.product", "USB Raw Device Access");
+	hal_device_property_set_string (d, "usbraw.device", device_file);
+
+out:
+	return d;
+}
+
+static gboolean
+usbraw_compute_udi (HalDevice *d)
+{
+	gchar udi[256];
+
+	hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi), "%s_usbraw",
+			      hal_device_property_get_string (d, "info.parent"));
+	hal_device_set_udi (d, udi);
+	hal_device_property_set_string (d, "info.udi", udi);
+
+	return TRUE;
+}
+
+
+/*--------------------------------------------------------------------------------------------------------------*/
+
+static HalDevice *
+video4linux_add (const gchar *sysfs_path, const gchar *device_file, HalDevice *physdev, const gchar *sysfs_path_in_devices)
+{
+	HalDevice *d;
+
+	d = NULL;
+
+	if (physdev == NULL || sysfs_path_in_devices == NULL)
+		goto out;
+
+	d = hal_device_new ();
+	hal_device_property_set_string (d, "linux.sysfs_path", sysfs_path);
+	hal_device_property_set_string (d, "info.parent", physdev->udi);
+	hal_device_property_set_string (d, "info.category", "video4linux");
+	hal_device_add_capability (d, "video4linux");
+	hal_device_property_set_string (d, "info.product", "Video Device");
+	hal_device_property_set_string (d, "video4linux.device", device_file);
+
+out:
+	return d;
+}
+
+static gboolean
+video4linux_compute_udi (HalDevice *d)
+{
+	gchar udi[256];
+
+	hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi), "%s_video4linux",
+			      hal_device_property_get_string (d, "info.parent"));
+	hal_device_set_udi (d, udi);
+	hal_device_property_set_string (d, "info.udi", udi);
+
+	return TRUE;
+}
+
+
+/*--------------------------------------------------------------------------------------------------------------*/
+
+static HalDevice *
+dvb_add (const gchar *sysfs_path, const gchar *device_file, HalDevice *physdev, const gchar *sysfs_path_in_devices)
+{
+	HalDevice *d;
+
+	d = NULL;
+
+	if (physdev == NULL || sysfs_path_in_devices == NULL)
+		goto out;
+
+	d = hal_device_new ();
+	hal_device_property_set_string (d, "linux.sysfs_path", sysfs_path);
+	hal_device_property_set_string (d, "info.parent", physdev->udi);
+	hal_device_property_set_string (d, "info.category", "dvb");
+	hal_device_add_capability (d, "dvb");
+	hal_device_property_set_string (d, "info.product", "DVB Device");
+	hal_device_property_set_string (d, "dvb.device", device_file);
+
+out:
+	return d;
+}
+
+static gboolean
+dvb_compute_udi (HalDevice *d)
+{
+	gchar udi[256];
+
+	hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi), "%s_dvb",
+			      hal_device_property_get_string (d, "info.parent"));
+	hal_device_set_udi (d, udi);
+	hal_device_property_set_string (d, "info.udi", udi);
+
+	return TRUE;
+}
+
+
 /*--------------------------------------------------------------------------------------------------------------*/
 
 static HalDevice *
@@ -453,96 +605,254 @@ sound_add (const gchar *sysfs_path, const gchar *device_file, HalDevice *physdev
 	int cardnum, devicenum;
 	char type;
 	const gchar *device;
+	gchar *device_id;
 	char aprocdir[256];
 	char buf[256];
 
 	d = NULL;
 
-	if (physdev == NULL || sysfs_path_in_devices == NULL || device_file == NULL) {
-		goto out;
-	}
+	if (device_file == NULL) 
+		goto out;	
 
 	d = hal_device_new ();
 	hal_device_property_set_string (d, "linux.sysfs_path", sysfs_path);
-	hal_device_property_set_string (d, "info.parent", physdev->udi);
-	hal_device_property_set_string (d, "info.category", "alsa");
-	hal_device_add_capability (d, "alsa");
-	hal_device_property_set_string (d, "alsa.physical_device", physdev->udi);
-
 	device = hal_util_get_last_element(sysfs_path);
-	if (sscanf (device, "controlC%d", &cardnum) == 1) {
-		hal_device_property_set_int (d, "alsa.card", cardnum);
-		hal_device_property_set_string (d, "alsa.type", "control");
 
-		snprintf (aprocdir, sizeof (aprocdir), "%s/asound/card%d", get_hal_proc_path (), cardnum);
-		hal_util_set_string_from_file (d, "alsa.card_id", aprocdir, "id");
+	if (physdev == NULL || sysfs_path_in_devices == NULL) {
+ 		/* handle global ALSA and OSS devices, these devices are for all ALSA/OSS Sound devices
+		   so we append them to /org/freedesktop/Hal/devices/computer */
+		hal_device_property_set_string (d, "info.parent", "/org/freedesktop/Hal/devices/computer");
 
-		snprintf (buf, sizeof (buf), "%s ALSA Control Device", 
-			  hal_device_property_get_string (d, "alsa.card_id"));
-		hal_device_property_set_string (d, "info.product", buf);
-
-	} else if (sscanf (device, "pcmC%dD%d%c", &cardnum, &devicenum, &type) == 3) {
-		gchar *device_id;
-
-		hal_device_property_set_int (d, "alsa.card", cardnum);
-		hal_device_property_set_int (d, "alsa.device", devicenum);
-
-		snprintf (aprocdir, sizeof (aprocdir), "%s/asound/card%d", get_hal_proc_path (), cardnum);
-		hal_util_set_string_from_file (d, "alsa.card_id", aprocdir, "id");
-
-		snprintf (aprocdir, sizeof (aprocdir), "%s/asound/card%d/pcm%d%c", 
-			  get_hal_proc_path (), cardnum, devicenum, type);
-		device_id = hal_util_grep_file (aprocdir, "info", "name: ", FALSE);
-		if (device_id != NULL) {
-			hal_device_property_set_string (d, "alsa.device_id", device_id);
-		}
-
-		if (type == 'p') {
-			hal_device_property_set_string (d, "alsa.type", "playback");
-			if (device_id != NULL) {
-				snprintf (buf, sizeof (buf), "%s ALSA Playback Device", device_id);
-				hal_device_property_set_string (d, "info.product", buf);
-			} else
-				hal_device_property_set_string (d, "info.product", "ALSA Playback Device");
-		} else if (type == 'c') {
-			hal_device_property_set_string (d, "alsa.type", "capture");
-			if (device_id != NULL) {
-				snprintf (buf, sizeof (buf), "%s ALSA Capture Device", device_id);
-				hal_device_property_set_string (d, "info.product", buf);
-			} else
-				  hal_device_property_set_string (d, "info.product", "ALSA Capture Device");
+		if (!strncmp (device, "timer", 5)){
+			/* handle global ALSA Timer device */
+			hal_device_property_set_string (d, "info.category", "alsa");
+			hal_device_add_capability (d, "alsa");
+			hal_device_property_set_string (d, "alsa.type", "timer");
+			hal_device_property_set_string (d, "info.product", "ALSA Timer Device");
+			hal_device_property_set_string (d, "alsa.device_file", device_file);
+		} else if (!strncmp (device, "sequencer", 9)){
+			/* handle global OSS sequencer devices */
+			hal_device_property_set_string (d, "info.category", "oss");
+			hal_device_add_capability (d, "oss");
+			hal_device_property_set_string (d, "oss.type", "sequencer");
+			hal_device_property_set_string (d, "info.product", "OSS Sequencer Device");
+			hal_device_property_set_string (d, "oss.device_file", device_file);
+		} else if (!strncmp (device, "seq", 3) && strlen(device) == 3) { 
+			/* handle global ALSA sequencer devices */
+			hal_device_property_set_string (d, "info.category", "alsa");
+			hal_device_add_capability (d, "alsa");
+			hal_device_property_set_string (d, "alsa.type", "sequencer");
+			hal_device_property_set_string (d, "info.product", "ALSA Sequencer Device");
+			hal_device_property_set_string (d, "alsa.device_file", device_file);	
 		} else {
-			hal_device_property_set_string (d, "alsa.type", "unknown");
-			if (device_id != NULL) {
-				snprintf (buf, sizeof (buf), "%s ALSA Device", device_id);
-				hal_device_property_set_string (d, "info.product", buf);
-			} else
-				  hal_device_property_set_string (d, "info.product", "ALSA Device");
+			goto error;
 		}
 	} else {
-		g_object_unref (d);
-		d = NULL;
-	}
+		/* handle ALSA and OSS devices with physdev link in sys */
+		if (sscanf (device, "controlC%d", &cardnum) == 1) {
+			
+			hal_device_property_set_string (d, "info.category", "alsa");
+			hal_device_add_capability (d, "alsa");
+			hal_device_property_set_string (d, "alsa.device_file", device_file);
+			hal_device_property_set_string (d, "info.parent", physdev->udi);
+			hal_device_property_set_string (d, "alsa.physical_device", physdev->udi);
+			hal_device_property_set_int (d, "alsa.card", cardnum);
+			hal_device_property_set_string (d, "alsa.type", "control");
+	
+			snprintf (aprocdir, sizeof (aprocdir), "%s/asound/card%d", get_hal_proc_path (), cardnum);
+			hal_util_set_string_from_file (d, "alsa.card_id", aprocdir, "id");
+	
+			snprintf (buf, sizeof (buf), "%s ALSA Control Device", 
+				hal_device_property_get_string (d, "alsa.card_id"));
+			hal_device_property_set_string (d, "info.product", buf);
+	
+		} else if (sscanf (device, "pcmC%dD%d%c", &cardnum, &devicenum, &type) == 3) {
+			
+			hal_device_property_set_string (d, "info.category", "alsa");
+			hal_device_add_capability (d, "alsa");
+			hal_device_property_set_string (d, "alsa.device_file", device_file);
+			hal_device_property_set_string (d, "info.parent", physdev->udi);
+			hal_device_property_set_string (d, "alsa.physical_device", physdev->udi);
+			hal_device_property_set_int (d, "alsa.card", cardnum);
+			hal_device_property_set_int (d, "alsa.device", devicenum);
+	
+			snprintf (aprocdir, sizeof (aprocdir), "%s/asound/card%d", get_hal_proc_path (), cardnum);
+			hal_util_set_string_from_file (d, "alsa.card_id", aprocdir, "id");
+	
+			snprintf (aprocdir, sizeof (aprocdir), "%s/asound/card%d/pcm%d%c", 
+				get_hal_proc_path (), cardnum, devicenum, type);
+			device_id = hal_util_grep_file (aprocdir, "info", "name: ", FALSE);
+			if (device_id != NULL) {
+				hal_device_property_set_string (d, "alsa.device_id", device_id);
+			}
+	
+			if (type == 'p') {
+				hal_device_property_set_string (d, "alsa.type", "playback");
+				if (device_id != NULL) {
+					snprintf (buf, sizeof (buf), "%s ALSA Playback Device", device_id);
+					hal_device_property_set_string (d, "info.product", buf);
+				} else
+					hal_device_property_set_string (d, "info.product", "ALSA Playback Device");
+			} else if (type == 'c') {
+				hal_device_property_set_string (d, "alsa.type", "capture");
+				if (device_id != NULL) {
+					snprintf (buf, sizeof (buf), "%s ALSA Capture Device", device_id);
+					hal_device_property_set_string (d, "info.product", buf);
+				} else
+					hal_device_property_set_string (d, "info.product", "ALSA Capture Device");
+			} else {
+				hal_device_property_set_string (d, "alsa.type", "unknown");
+				if (device_id != NULL) {
+					snprintf (buf, sizeof (buf), "%s ALSA Device", device_id);
+					hal_device_property_set_string (d, "info.product", buf);
+				} else
+					hal_device_property_set_string (d, "info.product", "ALSA Device");
+			}
+		} else if (!strncmp (device, "dsp", 3) || !strncmp (device, "adsp", 4) || 
+			   !strncmp (device, "midi", 4) || !strncmp (device, "amidi", 5) ||
+			   !strncmp (device, "audio", 5) || !strncmp (device, "mixer", 5)) {
+			
+			/* handle OSS-Devices */
+			ClassDevOSSDeviceTypes type;
+	
+			if (!strncmp (device, "dsp", 3)) {
+				if(sscanf (device, "dsp%d", &cardnum) != 1) cardnum = 0;
+				type = OSS_DEVICE_TYPE_DSP;
+			} else if (!strncmp (device, "adsp", 4)) {
+				if(sscanf (device, "adsp%d", &cardnum) != 1) cardnum = 0;
+				type = OSS_DEVICE_TYPE_ADSP;
+			} else if (!strncmp (device, "midi", 4)) {
+				if(sscanf (device, "midi%d", &cardnum) != 1) cardnum = 0;
+				type = OSS_DEVICE_TYPE_MIDI;
+			} else if (!strncmp (device, "amidi", 5)) {
+				if(sscanf (device, "amidi%d", &cardnum) != 1) cardnum = 0;
+				type = OSS_DEVICE_TYPE_AMIDI;
+			} else if (!strncmp (device, "audio", 5)) {
+				if(sscanf (device, "audio%d", &cardnum) != 1) cardnum = 0;
+				type = OSS_DEVICE_TYPE_AUDIO;
+			} else if (!strncmp (device, "mixer", 5)) {
+				if(sscanf (device, "mixer%d", &cardnum) != 1) cardnum = 0;
+				type = OSS_DEVICE_TYPE_MIXER;
+			} else {
+				cardnum = 0;
+				type = OSS_DEVICE_TYPE_UNKNOWN;
+			}
 
+			hal_device_property_set_string (d, "info.category", "oss");
+			hal_device_add_capability (d, "oss");
+			hal_device_property_set_string (d, "oss.device_file", device_file);
+			hal_device_property_set_string (d, "info.parent", physdev->udi);
+			hal_device_property_set_string (d, "oss.physical_device", physdev->udi);
+			hal_device_property_set_int (d, "oss.card", cardnum);
+	
+			snprintf (aprocdir, sizeof (aprocdir), "%s/asound/card%d", get_hal_proc_path (), cardnum);
+			hal_util_set_string_from_file (d, "oss.card_id", aprocdir, "id");
+	
+			snprintf (aprocdir, sizeof (aprocdir), "%s/asound/card%d/pcm0p", 
+				get_hal_proc_path (), cardnum);
+			device_id = hal_util_grep_file (aprocdir, "info", "name: ", FALSE);
+			if (device_id != NULL) {
+				hal_device_property_set_string (d, "oss.device_id", device_id);
+			} 
+
+			switch (type) { 
+				case OSS_DEVICE_TYPE_MIXER:
+					hal_device_property_set_string (d, "oss.type", "mixer");
+					if (device_id != NULL) 
+						snprintf (buf, sizeof (buf), "%s OSS Control Device", device_id); 
+					else
+						snprintf (buf, sizeof (buf), "%s OSS Control Device",
+						          hal_device_property_get_string (d, "oss.card_id"));
+					break;
+				case OSS_DEVICE_TYPE_DSP:
+				case OSS_DEVICE_TYPE_AUDIO:
+				case OSS_DEVICE_TYPE_ADSP:
+					if (type == OSS_DEVICE_TYPE_ADSP)
+						hal_device_property_set_int (d, "oss.device", 1);
+					else 
+						hal_device_property_set_int (d, "oss.device", 0);
+
+					hal_device_property_set_string (d, "oss.type", "pcm");
+					if (device_id != NULL) 
+						snprintf (buf, sizeof (buf), "%s OSS PCM Device", device_id); 
+					else
+						snprintf (buf, sizeof (buf), "%s OSS PCM Device",
+						          hal_device_property_get_string (d, "oss.card_id"));
+					break;
+				case OSS_DEVICE_TYPE_MIDI:
+				case OSS_DEVICE_TYPE_AMIDI:
+					if (type == OSS_DEVICE_TYPE_AMIDI)
+						hal_device_property_set_int (d, "oss.device", 1);
+					else
+						hal_device_property_set_int (d, "oss.device", 0);
+					hal_device_property_set_string (d, "oss.type", "midi");
+					if (device_id != NULL) 
+						snprintf (buf, sizeof (buf), "%s OSS MIDI Device", device_id); 
+					else
+						snprintf (buf, sizeof (buf), "%s OSS MIDI Device",
+						          hal_device_property_get_string (d, "oss.card_id"));
+					break;
+				case OSS_DEVICE_TYPE_UNKNOWN:
+				default:
+					hal_device_property_set_string (d, "oss.type", "unknown");
+					if (device_id != NULL) 
+						snprintf (buf, sizeof (buf), "%s OSS Device", device_id); 
+					else
+						snprintf (buf, sizeof (buf), "%s OSS Device",
+						          hal_device_property_get_string (d, "oss.card_id"));
+					break;
+			}
+			hal_device_property_set_string (d, "info.product", buf);
+		}
+		else {
+			goto error;
+		}
+	}
 out:
 	return d;
+
+error: 
+	g_object_unref (d);
+	d = NULL;
+	return d;
 }
 
 static gboolean
 sound_compute_udi (HalDevice *d)
 {
 	gchar udi[256];
-	int cardnum, devicenum;
 
-	cardnum = hal_device_property_get_int (d, "alsa.card");
-	devicenum = hal_device_property_get_int (d, "alsa.device");
-
-	hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
-			      "%s_alsa_%s_%i_%i",
-			      hal_device_property_get_string (d, "info.parent"),
-			      hal_device_property_get_string (d, "alsa.type"),
-			      hal_device_property_get_int (d, "alsa.card"),
-			      hal_device_property_get_int (d, "alsa.device"));
+	if (hal_device_has_property(d, "alsa.card")) {
+		/* don't include card number as it may not be persistent across reboots */
+		hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
+				      "%s_alsa_%s_%i",
+				      hal_device_property_get_string (d, "info.parent"),
+				      hal_device_property_get_string (d, "alsa.type"),
+				      hal_device_property_get_int (d, "alsa.device"));
+	} else if (hal_device_has_property(d, "oss.card")) {
+		/* don't include card number as it may not be persistent across reboots */
+		hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
+				      "%s_oss_%s_%i",
+				      hal_device_property_get_string (d, "info.parent"),
+				      hal_device_property_get_string (d, "oss.type"),
+				      hal_device_property_get_int (d, "oss.device"));
+	} else if (hal_device_has_property(d, "alsa.type")) {
+		/* handle global ALAS devices */
+		hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
+				      "%s_alsa_%s",
+				      hal_device_property_get_string (d, "info.parent"),
+				      hal_device_property_get_string (d, "alsa.type"));
+	} else if (hal_device_has_property(d, "oss.type")) {
+		/* handle global OSS devices */
+		hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
+				      "%s_oss_%s",
+				      hal_device_property_get_string (d, "info.parent"),
+				      hal_device_property_get_string (d, "oss.type"));
+	} else {
+		/* fallback */
+		hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi), "%s_sound_unknown",
+				      hal_device_property_get_string (d, "info.parent"));
+	} 
 	hal_device_set_udi (d, udi);
 	hal_device_property_set_string (d, "info.udi", udi);
 
@@ -610,16 +920,16 @@ serial_add (const gchar *sysfs_path, const gchar *device_file, HalDevice *physde
 		hal_device_property_set_string (d, "info.product", "Serial Port");
 	}
 
-	/* TODO: run prober to see if there actually is a physical
-	 *       serial port if possible (on my T41, the kernel
-	 *       exports ttyS0 but there is no real port on the
-	 *       laptop; only on the docking station)
-	 */
-
 out:
 	return d;
 }
 
+static const gchar *
+serial_get_prober (HalDevice *d)
+{
+	return "hald-probe-serial";
+}
+
 static gboolean
 serial_compute_udi (HalDevice *d)
 {
@@ -688,6 +998,53 @@ tape_compute_udi (HalDevice *d)
 
 /*--------------------------------------------------------------------------------------------------------------*/
 
+static HalDevice *
+mmc_host_add (const gchar *sysfs_path, const gchar *device_file, HalDevice *physdev, const gchar *sysfs_path_in_devices)
+{
+	HalDevice *d;
+	gint host_num;
+	const gchar *last_elem;
+
+	d = NULL;
+
+	if (physdev == NULL || sysfs_path_in_devices == NULL) {
+		goto out;
+	}
+
+	d = hal_device_new ();
+	hal_device_property_set_string (d, "linux.sysfs_path", sysfs_path);
+	hal_device_property_set_string (d, "linux.sysfs_path_device", sysfs_path_in_devices);
+
+	hal_device_property_set_string (d, "info.parent", physdev->udi);
+
+	hal_device_property_set_string (d, "info.category", "mmc_host");
+	hal_device_add_capability (d, "mmc_host");
+
+	hal_device_property_set_string (d, "info.product", "MMC/SD Host Adapter");
+
+	last_elem = hal_util_get_last_element (sysfs_path);
+	sscanf (last_elem, "mmc%d", &host_num);
+	hal_device_property_set_int (d, "mmc_host.host", host_num);
+
+out:
+	return d;
+}
+
+static gboolean
+mmc_host_compute_udi (HalDevice *d)
+{
+	gchar udi[256];
+
+	hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
+			      "%s_mmc_host",
+			      hal_device_property_get_string (d, "info.parent"));
+	hal_device_set_udi (d, udi);
+	hal_device_property_set_string (d, "info.udi", udi);
+	return TRUE;
+}
+
+/*--------------------------------------------------------------------------------------------------------------*/
+
 static gboolean
 classdev_remove (HalDevice *d)
 {
@@ -741,6 +1098,16 @@ static ClassDevHandler classdev_handler_net =
 	.remove       = classdev_remove
 };
 
+static ClassDevHandler classdev_handler_scsi_generic =
+{ 
+	.subsystem    = "scsi_generic",
+	.add          = scsi_generic_add,
+	.get_prober   = NULL,
+	.post_probing = NULL,
+	.compute_udi  = scsi_generic_compute_udi,
+	.remove       = classdev_remove
+};
+
 static ClassDevHandler classdev_handler_scsi_host = 
 { 
 	.subsystem    = "scsi_host",
@@ -761,6 +1128,36 @@ static ClassDevHandler classdev_handler_usbclass =
 	.remove       = classdev_remove
 };
 
+static ClassDevHandler classdev_handler_usbraw =
+{ 
+	.subsystem    = "usb_device",
+	.add          = usbraw_add,
+	.get_prober   = NULL,
+	.post_probing = NULL,
+	.compute_udi  = usbraw_compute_udi,
+	.remove       = classdev_remove
+};
+
+static ClassDevHandler classdev_handler_video4linux =
+{ 
+	.subsystem    = "video4linux",
+	.add          = video4linux_add,
+	.get_prober   = NULL,
+	.post_probing = NULL,
+	.compute_udi  = video4linux_compute_udi,
+	.remove       = classdev_remove
+};
+
+static ClassDevHandler classdev_handler_dvb =
+{ 
+	.subsystem    = "dvb",
+	.add          = dvb_add,
+	.get_prober   = NULL,
+	.post_probing = NULL,
+	.compute_udi  = dvb_compute_udi,
+	.remove       = classdev_remove
+};
+
 static ClassDevHandler classdev_handler_sound = 
 { 
 	.subsystem    = "sound",
@@ -775,7 +1172,7 @@ static ClassDevHandler classdev_handler_serial =
 { 
 	.subsystem    = "tty",
 	.add          = serial_add,
-	.get_prober   = NULL,
+	.get_prober   = serial_get_prober,
 	.post_probing = NULL,
 	.compute_udi  = serial_compute_udi,
 	.remove       = classdev_remove
@@ -801,16 +1198,31 @@ static ClassDevHandler classdev_handler_tape390 =
 	.remove       = classdev_remove
 };
 
+static ClassDevHandler classdev_handler_mmc_host =
+{
+	.subsystem    = "mmc_host",
+	.add          = mmc_host_add,
+	.get_prober   = NULL,
+	.post_probing = NULL,
+	.compute_udi  = mmc_host_compute_udi,
+	.remove       = classdev_remove
+};
+
 static ClassDevHandler *classdev_handlers[] = {
 	&classdev_handler_input,
 	&classdev_handler_bluetooth,
 	&classdev_handler_net,
+	&classdev_handler_scsi_generic,
 	&classdev_handler_scsi_host,
 	&classdev_handler_usbclass,
+	&classdev_handler_usbraw,
+	&classdev_handler_video4linux,
+	&classdev_handler_dvb,
 	&classdev_handler_sound,
 	&classdev_handler_serial,
 	&classdev_handler_tape,
 	&classdev_handler_tape390,
+	&classdev_handler_mmc_host,
 	NULL
 };
 
@@ -871,13 +1283,14 @@ out:
 }
 
 static void 
-add_classdev_probing_helper_done (HalDevice *d, gboolean timed_out, gint return_code, 
-				  gpointer data1, gpointer data2, HalHelperData *helper_data)
+add_classdev_probing_helper_done (HalDevice *d, guint32 exit_type, 
+                                  gint return_code, char **error,
+                                  gpointer data1, gpointer data2) 
 {
 	void *end_token = (void *) data1;
 	ClassDevHandler *handler = (ClassDevHandler *) data2;
 
-	HAL_INFO (("entering; timed_out=%d, return_code=%d", timed_out, return_code));
+	HAL_INFO (("entering; exit_type=%d, return_code=%d", exit_type, return_code));
 
 	if (d == NULL) {
 		HAL_INFO (("Device object already removed"));
@@ -886,7 +1299,7 @@ add_classdev_probing_helper_done (HalDevice *d, gboolean timed_out, gint return_
 	}
 
 	/* Discard device if probing reports failure */
-	if (return_code != 0) {
+	if (exit_type != HALD_RUN_SUCCESS || return_code != 0) {
 		hal_device_store_remove (hald_get_tdl (), d);
 		g_object_unref (d);
 		hotplug_event_end (end_token);
@@ -939,20 +1352,16 @@ classdev_callouts_preprobing_done (HalDevice *d, gpointer userdata1, gpointer us
 		prober = NULL;
 	if (prober != NULL) {
 		/* probe the device */
-		if (hal_util_helper_invoke (prober, NULL, d, (gpointer) end_token, 
-					    (gpointer) handler, add_classdev_probing_helper_done, 
-					    HAL_HELPER_TIMEOUT) == NULL) {
-			hal_device_store_remove (hald_get_tdl (), d);
-			g_object_unref (d);
-			hotplug_event_end (end_token);
-		}
-		goto out;
+		hald_runner_run(d, 
+		                    prober, NULL, 
+		                    HAL_HELPER_TIMEOUT, 
+		                    add_classdev_probing_helper_done,
+		                    (gpointer) end_token, (gpointer) handler);
 	} else {
 		add_classdev_after_probing (d, handler, end_token);
-		goto out;
 	}
 out:
-	;
+  ;
 }
 
 void
@@ -963,6 +1372,12 @@ hotplug_event_begin_add_classdev (const gchar *subsystem, const gchar *sysfs_pat
 
 	HAL_INFO (("class_add: subsys=%s sysfs_path=%s dev=%s physdev=0x%08x", subsystem, sysfs_path, device_file, physdev));
 
+	/* update driver property of the physical device, cause manual driver bind/unbind
+	 * may change change this without sending events for the bus device
+	 */
+	if (physdev != NULL)
+		hal_util_set_driver (physdev, "info.linux.driver", sysfs_path_in_devices);
+
 	if (physdev != NULL && hal_device_property_get_bool (physdev, "info.ignore")) {
 		HAL_INFO (("Ignoring class_add since physdev has info.ignore==TRUE"));
 		hotplug_event_end (end_token);
