@@ -1,11 +1,11 @@
 /***************************************************************************
- * CVSID: $Id: physdev.c,v 1.17 2005/06/27 19:13:03 david Exp $
+ * CVSID: $Id: physdev.c,v 1.26 2006/02/02 13:06:32 kay Exp $
  *
  * physdev.c : Handling of physical kernel devices 
  *
  * Copyright (C) 2004 David Zeuthen, <david@fubar.dk>
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  **************************************************************************/
 
@@ -31,7 +31,7 @@
 #include <string.h>
 #include <mntent.h>
 #include <errno.h>
-#include <sys/types.h>
+#include <stdint.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -56,8 +56,6 @@
 
 #include "ids.h"
 
-#include "pcmcia_utils.h"
-
 #include "osspec_linux.h"
 
 /*--------------------------------------------------------------------------------------------------------------*/
@@ -483,6 +481,52 @@ pnp_compute_udi (HalDevice *d)
 
 /*--------------------------------------------------------------------------------------------------------------*/
 
+static HalDevice *
+platform_add (const gchar *sysfs_path, HalDevice *parent)
+{
+	HalDevice *d;
+	const gchar *dev_id;
+	gchar buf[64];
+
+	d = hal_device_new ();
+	hal_device_property_set_string (d, "linux.sysfs_path", sysfs_path);
+	hal_device_property_set_string (d, "linux.sysfs_path_device", sysfs_path);
+	hal_device_property_set_string (d, "info.bus", "platform");
+	if (parent != NULL) {
+		hal_device_property_set_string (d, "info.parent", parent->udi);
+	} else {
+		hal_device_property_set_string (d, "info.parent", "/org/freedesktop/Hal/devices/computer");
+	}
+
+	hal_util_set_driver (d, "info.linux.driver", sysfs_path);
+
+	dev_id = hal_util_get_last_element (sysfs_path);
+
+	hal_device_property_set_string (d, "platform.id", dev_id);
+
+	g_snprintf (buf, sizeof (buf), "Platform Device (%s)", hal_device_property_get_string (d, "platform.id"));
+	hal_device_property_set_string (d, "info.product", buf);
+
+	return d;
+}
+
+static gboolean
+platform_compute_udi (HalDevice *d)
+{
+	gchar udi[256];
+
+	hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
+			      "/org/freedesktop/Hal/devices/platform_%s",
+			      hal_device_property_get_string (d, "platform.id"));
+	hal_device_set_udi (d, udi);
+	hal_device_property_set_string (d, "info.udi", udi);
+
+	return TRUE;
+
+}
+
+/*--------------------------------------------------------------------------------------------------------------*/
+
 static HalDevice *
 serio_add (const gchar *sysfs_path, HalDevice *parent)
 {
@@ -534,6 +578,8 @@ pcmcia_add (const gchar *sysfs_path, HalDevice *parent)
 	HalDevice *d;
 	const gchar *bus_id;
 	guint socket, function;
+	const char *prod_id1;
+	const char *prod_id2;
 
 	d = hal_device_new ();
 	hal_device_property_set_string (d, "linux.sysfs_path", sysfs_path);
@@ -553,61 +599,35 @@ pcmcia_add (const gchar *sysfs_path, HalDevice *parent)
 	sscanf (bus_id, "%d.%d", &socket, &function);
 	hal_device_property_set_int (d, "pcmcia.socket_number", socket);
 
-	/* TODO: need to read this from sysfs instead of relying on stab */
-	{
-		pcmcia_card_info *info;
+	hal_util_set_string_from_file (d, "pcmcia.prod_id1", sysfs_path, "prod_id1");
+	hal_util_set_string_from_file (d, "pcmcia.prod_id2", sysfs_path, "prod_id2");
+	hal_util_set_string_from_file (d, "pcmcia.prod_id3", sysfs_path, "prod_id3");
+	hal_util_set_string_from_file (d, "pcmcia.prod_id4", sysfs_path, "prod_id4");
 
-		info = pcmcia_card_info_get (socket);
-		if (info != NULL) {
-			const char *type;
+	hal_util_set_int_from_file (d, "pcmcia.manf_id", sysfs_path, "manf_id", 16);
+	hal_util_set_int_from_file (d, "pcmcia.card_id", sysfs_path, "card_id", 16);
+	hal_util_set_int_from_file (d, "pcmcia.func_id", sysfs_path, "func_id", 16);
 
-			if (info->productid_1 != NULL && strlen (info->productid_1) > 0)
-				hal_device_property_set_string (d, "pcmcia.productid_1", info->productid_1);
-			else
-				hal_device_property_set_string (d, "pcmcia.productid_1", "");
-			if (info->productid_2 != NULL && strlen (info->productid_2) > 0)
-				hal_device_property_set_string (d, "pcmcia.productid_2", info->productid_2);
-			else
-				hal_device_property_set_string (d, "pcmcia.productid_2", "");
-			if (info->productid_3 != NULL && strlen (info->productid_3) > 0)
-				hal_device_property_set_string (d, "pcmcia.productid_3", info->productid_3);
-			else
-				hal_device_property_set_string (d, "pcmcia.productid_3", "");
-			if (info->productid_4 != NULL && strlen (info->productid_4) > 0)
-				hal_device_property_set_string (d, "pcmcia.productid_4", info->productid_4);
-			else
-				hal_device_property_set_string (d, "pcmcia.productid_4", "");
+	prod_id1 = hal_device_property_get_string (d, "pcmcia.prod_id1");
+	prod_id2 = hal_device_property_get_string (d, "pcmcia.prod_id2");
 
-			if ((type = pcmcia_card_type_string_from_type (info->type)))
-				hal_device_property_set_string (d, "pcmcia.function", type);
-			else
-				hal_device_property_set_string (d, "pcmcia.function", "");
-
-			hal_device_property_set_int (d, "pcmcia.manfid1", info->manfid_1);
-			hal_device_property_set_int (d, "pcmcia.manfid2", info->manfid_2);
-
-			/* Provide best-guess of vendor, goes in Vendor property */
-			if (info->productid_1 != NULL) {
-				hal_device_property_set_string (d, "info.vendor", info->productid_1);
-			} else {
-				gchar buf[50];
-				g_snprintf (buf, sizeof(buf), "Unknown (0x%04x)", info->manfid_1);
-				hal_device_property_set_string (d, "info.vendor", buf);
-			}
-
-			/* Provide best-guess of name, goes in Product property */
-			if (info->productid_2 != NULL) {
-				hal_device_property_set_string (d, "info.product", info->productid_2);
-			} else {
-				gchar buf[50];
-				g_snprintf (buf, sizeof(buf), "Unknown (0x%04x)", info->manfid_2);
-				hal_device_property_set_string (d, "info.product", buf);
-			}
-
-			pcmcia_card_info_free (info);
-		}
+	/* Provide best-guess of vendor, goes in Vendor property */
+	if (prod_id1 != NULL) {
+		hal_device_property_set_string (d, "info.vendor", prod_id1);
+	} else {
+		char buf[50];
+		g_snprintf (buf, sizeof(buf), "Unknown (0x%04x)", hal_device_property_get_int (d, "pcmcia.manf_id"));
+		hal_device_property_set_string (d, "info.vendor", buf);
 	}
 
+	/* Provide best-guess of name, goes in Product property */
+	if (prod_id2 != NULL) {
+		hal_device_property_set_string (d, "info.product", prod_id2);
+	} else {
+		char buf[50];
+		g_snprintf (buf, sizeof(buf), "Unknown (0x%04x)", hal_device_property_get_int (d, "pcmcia.card_id"));
+		hal_device_property_set_string (d, "info.product", buf);
+	}
 
 	return d;
 }
@@ -635,6 +655,7 @@ scsi_add (const gchar *sysfs_path, HalDevice *parent)
 	HalDevice *d;
 	const gchar *bus_id;
 	gint host_num, bus_num, target_num, lun_num;
+	int type;
 
 	if (parent == NULL) {
 		d = NULL;
@@ -656,9 +677,35 @@ scsi_add (const gchar *sysfs_path, HalDevice *parent)
 
 	hal_util_set_driver (d, "info.linux.driver", sysfs_path);
 
-	/* guestimate product name */
 	hal_device_property_set_string (d, "info.product", "SCSI Device");
 
+	hal_util_set_string_from_file (d, "scsi.model", sysfs_path, "model");
+	hal_util_set_string_from_file (d, "scsi.vendor", sysfs_path, "vendor");
+	hal_util_get_int_from_file (sysfs_path, "type", &type, 0);
+	switch (type) {
+	case 0:
+		/* Disk */
+	case 14:
+		/* TYPE_RBC (Reduced Block Commands)
+		 * Simple Direct Access Device, set it to disk
+		 * (some Firewire Disks use it)
+		 */
+		hal_device_property_set_string (d, "scsi.type", "disk");
+		break;
+	case 1:
+		/* Tape */
+		hal_device_property_set_string (d, "scsi.type", "tape");
+		break;
+	case 4:
+		/* WORM */
+	case 5:
+		/* CD-ROM */
+		hal_device_property_set_string (d, "scsi.type", "cdrom");
+		break;
+	default:
+		hal_device_property_set_string (d, "scsi.type", "unknown");
+	}
+
 out:
 	return d;
 }
@@ -892,7 +939,7 @@ ccw_add_tape_properties (HalDevice *d, const gchar *sysfs_path)
 	if (!hal_util_get_int_from_file (sysfs_path, "medium_state",
 					&medium_state, 10))
 		return;
-	hal_device_property_set_string (d, "tape.ccw.medium_state",
+	hal_device_property_set_string (d, "ccw.tape.medium_state",
 					state_text[medium_state]);
 }
 
@@ -1062,7 +1109,7 @@ static inline void
 ccwgroup_add_lcs_properties (HalDevice *d, const gchar *sysfs_path)
 {
 	hal_util_set_int_from_file (d, "ccwgroup.lcs.portnumber", sysfs_path,
-				    "portno", 2);
+				    "portno", 10);
 	hal_util_set_string_from_file (d, "ccwgroup.lcs.type", sysfs_path,
 				       "type");
 	hal_util_set_int_from_file (d, "ccwgroup.lcs.lancmd_timeout",
@@ -1200,6 +1247,51 @@ iucv_compute_udi (HalDevice *d)
 
 /*--------------------------------------------------------------------------------------------------------------*/
 
+static HalDevice *
+pseudo_add (const gchar *sysfs_path, HalDevice *parent)
+{
+	HalDevice *d;
+	const gchar *dev_id;
+	gchar buf[64];
+
+	d = hal_device_new ();
+	hal_device_property_set_string (d, "linux.sysfs_path", sysfs_path);
+	hal_device_property_set_string (d, "linux.sysfs_path_device", sysfs_path);
+	hal_device_property_set_string (d, "info.bus", "pseudo");
+	if (parent != NULL) {
+		hal_device_property_set_string (d, "info.parent", parent->udi);
+	} else {
+		hal_device_property_set_string (d, "info.parent", "/org/freedesktop/Hal/devices/computer");
+	}
+
+	hal_util_set_driver (d, "info.linux.driver", sysfs_path);
+
+	dev_id = hal_util_get_last_element (sysfs_path);
+	hal_device_property_set_string (d, "pseudo.id", dev_id);
+
+	g_snprintf (buf, sizeof (buf), "SCSI Debug Device (%s)", hal_device_property_get_string (d, "pseudo.id"));
+	hal_device_property_set_string (d, "info.product", buf);
+
+	return d;
+}
+
+static gboolean
+pseudo_compute_udi (HalDevice *d)
+{
+	gchar udi[256];
+
+	hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
+			      "/org/freedesktop/Hal/devices/pseudo",
+			      hal_device_property_get_string (d, "platform.id"));
+	hal_device_set_udi (d, udi);
+	hal_device_property_set_string (d, "info.udi", udi);
+
+	return TRUE;
+
+}
+
+/*--------------------------------------------------------------------------------------------------------------*/
+
 static gboolean
 physdev_remove (HalDevice *d)
 {
@@ -1244,6 +1336,13 @@ static PhysDevHandler physdev_handler_pnp = {
 	.remove      = physdev_remove
 };
 
+static PhysDevHandler physdev_handler_platform = {
+	.subsystem   = "platform",
+	.add         = platform_add,
+	.compute_udi = platform_compute_udi,
+	.remove      = physdev_remove
+};
+
 static PhysDevHandler physdev_handler_serio = { 
 	.subsystem   = "serio",
 	.add         = serio_add,
@@ -1279,8 +1378,7 @@ static PhysDevHandler physdev_handler_ieee1394 = {
 	.remove      = physdev_remove
 };
 
-
-/* s390 specific busses */	
+/* s390 specific busses */
 static PhysDevHandler physdev_handler_ccw = {
 	.subsystem   = "ccw",
 	.add         = ccw_add,
@@ -1302,11 +1400,20 @@ static PhysDevHandler physdev_handler_iucv = {
 	.remove      = physdev_remove
 };
 
+/* SCSI debug, to test thousends of fake devices */
+static PhysDevHandler physdev_handler_pseudo = {
+	.subsystem   = "pseudo",
+	.add         = pseudo_add,
+	.compute_udi = pseudo_compute_udi,
+	.remove      = physdev_remove
+};
+
 static PhysDevHandler *phys_handlers[] = {
 	&physdev_handler_pci,
 	&physdev_handler_usb,
 	&physdev_handler_ide,
 	&physdev_handler_pnp,
+	&physdev_handler_platform,
 	&physdev_handler_serio,
 	&physdev_handler_pcmcia,
 	&physdev_handler_scsi,
@@ -1315,6 +1422,7 @@ static PhysDevHandler *phys_handlers[] = {
 	&physdev_handler_ccw,
 	&physdev_handler_ccwgroup,
 	&physdev_handler_iucv,
+	&physdev_handler_pseudo,
 	NULL
 };
 
