@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: pmu.c,v 1.11 2005/05/11 17:46:32 david Exp $
+ * CVSID: $Id: pmu.c,v 1.20 2006/01/29 17:07:54 hughsient Exp $
  *
  * Copyright (C) 2005 Sjoerd Simons <sjoerd at luon.net>
  * Copyright (C) 2005 David Zeuthen, Red Hat Inc., <davidz@redhat.com>
@@ -16,15 +16,19 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  **************************************************************************/
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
-
+ 
+#include <stdio.h>
+#include <stdlib.h>
+#include <fcntl.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "../device_info.h"
 #include "../logger.h"
@@ -39,7 +43,8 @@
 enum {
 	PMU_TYPE_BATTERY,
 	PMU_TYPE_AC_ADAPTER,
-	PMU_TYPE_LID_BUTTON
+	PMU_TYPE_LID_BUTTON,
+	PMU_TYPE_LAPTOP_PANEL
 };
 
 
@@ -54,20 +59,26 @@ typedef struct PMUDevHandler_s
 
 
 /* defines from the kernel PMU driver (include/linux/pmu.h) */
-#define PMU_BATT_PRESENT  0x00000001
-#define PMU_BATT_CHARGING 0x00000002
-#define PMU_BATT_TYPE_MASK  0x000000f0
-#define PMU_BATT_TYPE_SMART 0x00000010	  /* Smart battery */
-#define PMU_BATT_TYPE_HOOPER  0x00000020  /* 3400/3500 */
-#define PMU_BATT_TYPE_COMET 0x00000030	  /* 2400 */
+#define PMU_BATT_PRESENT	0x00000001
+#define PMU_BATT_CHARGING	0x00000002
+#define PMU_BATT_TYPE_MASK	0x000000f0
+#define PMU_BATT_TYPE_SMART	0x00000010	/* Smart battery */
+#define PMU_BATT_TYPE_HOOPER	0x00000020	/* 3400/3500 */
+#define PMU_BATT_TYPE_COMET	0x00000030	/* 2400 */
+
+#define PMU_POLL_INTERVAL	2000
+
+#define PMUDEV			"/dev/pmu"
 
-#define PMU_POLL_INTERVAL 2000
 
 static gboolean
 battery_refresh (HalDevice *d, PMUDevHandler *handler)
 {
 	const char *path;
 	int flags;
+	int last_full;
+	int remaining_time;
+	int remaining_percentage;
 
 	path = hal_device_property_get_string (d, "linux.pmu_path");
 	if (path == NULL)
@@ -113,15 +124,32 @@ battery_refresh (HalDevice *d, PMUDevHandler *handler)
 		else
 			hal_device_property_set_int (d, "battery.charge_level.rate", -current);
 
+		current = hal_device_property_get_int (d, "battery.charge_level.current");
+		last_full = hal_device_property_get_int (d, "battery.charge_level.last_full");
+
 		/* TODO: could read some pmu file? */
-		hal_device_property_set_int (d, "battery.remaining_time", 
-					     util_compute_time_remaining (
-						     d->udi,
-						     hal_device_property_get_int (d, "battery.charge_level.rate"),
-						     hal_device_property_get_int (d, "battery.charge_level.current"),
-						     hal_device_property_get_int (d, "battery.charge_level.last_full"),
-						     hal_device_property_get_bool (d, "battery.rechargeable.is_discharging"),
-						     hal_device_property_get_bool (d, "battery.rechargeable.is_charging")));
+		remaining_time = util_compute_time_remaining (
+					d->udi,
+					hal_device_property_get_int (d, "battery.charge_level.rate"),
+					current,
+					last_full,
+					hal_device_property_get_bool (d, "battery.rechargeable.is_discharging"),
+					hal_device_property_get_bool (d, "battery.rechargeable.is_charging"),
+					hal_device_property_get_bool (d, "battery.remaining_time.calculate_per_time"));
+		remaining_percentage = util_compute_percentage_charge (d->udi, current, last_full);
+		/*
+		 * Only set keys if no error (signified with negative return value)
+		 * Scrict checking is needed to ensure that the values presented by HAL
+		 * are 100% acurate.
+		 */
+		if (remaining_time > 0)
+			hal_device_property_set_int (d, "battery.remaining_time", remaining_time);
+		else
+			hal_device_property_remove (d, "battery.remaining_time");
+		if (remaining_percentage > 0)
+			hal_device_property_set_int (d, "battery.charge_level.percentage", remaining_percentage);
+		else
+			hal_device_property_remove (d, "battery.charge_level.percentage");
 
 		device_property_atomic_update_end ();
 	} else {
@@ -167,7 +195,32 @@ lid_button_refresh (HalDevice *d, PMUDevHandler *handler)
 	hal_device_property_set_bool (d, "button.has_state", TRUE);
 	hal_device_property_set_bool (d, "button.state.value", FALSE); 
 
-        /* assume lid is open, polling will tell us otherwise (TODO: figure out initial state) */
+	/* assume lid is open, polling will tell us otherwise 
+	 * (TODO: figure out initial state)
+	 */
+	return TRUE;
+}
+
+/** Refreshes a laptop screen connected to a PMU controller.
+ *  This is much simpler than ACPI as we have a *standard* ioctl to use.
+ *
+ *  @param	d		The hal device
+ *  @param	handler		What to do
+ */
+static gboolean
+laptop_panel_refresh (HalDevice *d, PMUDevHandler *handler)
+{
+	hal_device_property_set_string (d, "info.category", "laptop_panel");
+	hal_device_property_set_string (d, "info.product", "Apple Laptop Panel");
+
+	hal_device_property_set_string (d, "laptop_panel.access_method", "pmu");
+	/*
+	 * We can set laptop_panel.num_levels as it will not change, 
+	 * all powerbooks have 15 steps for brightness, where state 0
+	 * is backlight disable.
+	 */
+	hal_device_property_set_int (d, "laptop_panel.num_levels", 14);
+	hal_device_add_capability (d, "laptop_panel");
 	return TRUE;
 }
 
@@ -182,6 +235,17 @@ pmu_lid_compute_udi (HalDevice *d, PMUDevHandler *handler)
 	return TRUE;
 }
 
+static gboolean
+pmu_laptop_panel_compute_udi (HalDevice *d, PMUDevHandler *handler)
+{
+	gchar udi[256];
+	hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
+			      "/org/freedesktop/Hal/devices/pmu_lcd");
+	hal_device_set_udi (d, udi);
+	hal_device_property_set_string (d, "info.udi", udi);
+	return TRUE;
+}
+
 static gboolean
 pmu_poll (gpointer data)
 {
@@ -221,6 +285,23 @@ pmu_poll (gpointer data)
 	return TRUE;
 }
 
+/** Synthesizes a *specific* PMU object.
+ *
+ *  @param	fullpath	The PMU path, e.g. "/dev/pmu/info"
+ *  @param	pmu_type	The type of device, e.g. PMU_TYPE_BATTERY
+ */
+static void
+pmu_synthesize_item (const gchar *fullpath, int pmu_type)
+{
+	HotplugEvent *hotplug_event;
+	HAL_INFO (("Processing %s", fullpath));
+	hotplug_event = g_new0 (HotplugEvent, 1);
+	hotplug_event->action = HOTPLUG_ACTION_ADD;
+	hotplug_event->type = HOTPLUG_EVENT_PMU;
+	g_strlcpy (hotplug_event->pmu.pmu_path, fullpath, sizeof (hotplug_event->pmu.pmu_path));
+	hotplug_event->acpi.acpi_type = pmu_type;
+	hotplug_event_enqueue (hotplug_event);
+}
 
 /** Scan the data structures exported by the kernel and add hotplug
  *  events for adding PMU objects.
@@ -234,7 +315,6 @@ pmu_synthesize_hotplug_events (void)
 	gboolean ret;
 	HalDevice *computer;
 	gchar path[HAL_PATH_MAX];
-	HotplugEvent *hotplug_event;
 	GError *error;
 	GDir *dir;
 	gboolean has_battery_bays;
@@ -255,42 +335,25 @@ pmu_synthesize_hotplug_events (void)
 	}
 
 	/* Set appropriate properties on the computer object */
-	hal_device_property_set_bool (computer, "power_management.is_enabled", TRUE);
 	hal_device_property_set_string (computer, "power_management.type", "pmu");
 
 	/* AC Adapter */
 	snprintf (path, sizeof (path), "%s/pmu/info", get_hal_proc_path ());
-	hotplug_event = g_new0 (HotplugEvent, 1);
-	hotplug_event->action = HOTPLUG_ACTION_ADD;
-	hotplug_event->type = HOTPLUG_EVENT_PMU;
-	g_strlcpy (hotplug_event->pmu.pmu_path, path, sizeof (hotplug_event->pmu.pmu_path));
-	hotplug_event->pmu.pmu_type = PMU_TYPE_AC_ADAPTER;
-	hotplug_event_enqueue (hotplug_event);
+	pmu_synthesize_item (path, PMU_TYPE_AC_ADAPTER);
 
 	error = NULL;
 	snprintf (path, sizeof (path), "%s/pmu", get_hal_proc_path ());
 	dir = g_dir_open (path, 0, &error);
 	if (dir != NULL) {
 		const gchar *f;
-			
 		while ((f = g_dir_read_name (dir)) != NULL) {
-			HotplugEvent *hotplug_event;
 			gchar buf[HAL_PATH_MAX];
 			int battery_num;
 
 			snprintf (buf, sizeof (buf), "%s/pmu/%s", get_hal_proc_path (), f);
 			if (sscanf (f, "battery_%d", &battery_num) == 1) {
-				HAL_INFO (("Processing %s", buf));
-
 				has_battery_bays = TRUE;
-				
-				hotplug_event = g_new0 (HotplugEvent, 1);
-				hotplug_event->action = HOTPLUG_ACTION_ADD;
-				hotplug_event->type = HOTPLUG_EVENT_PMU;
-				g_strlcpy (hotplug_event->pmu.pmu_path, buf, sizeof (hotplug_event->pmu.pmu_path));
-				hotplug_event->pmu.pmu_type = PMU_TYPE_BATTERY;
-				
-				hotplug_event_enqueue (hotplug_event);
+				pmu_synthesize_item (buf, PMU_TYPE_BATTERY);
 			}
 			
 		}
@@ -302,17 +365,20 @@ pmu_synthesize_hotplug_events (void)
 	/* close directory */
 	g_dir_close (dir);
 
-	/* FIXME: Bah, here we need to make another assumption -
-	 * namely that there is a lid button, if, and only if, the
-	 * machine has got battery bays
+	/* We need to make another assumption - that there is a lid button,
+	 * if, and only if, the machine has got battery bays
 	 */
 	if (has_battery_bays) {
-		hotplug_event = g_new0 (HotplugEvent, 1);
-		hotplug_event->action = HOTPLUG_ACTION_ADD;
-		hotplug_event->type = HOTPLUG_EVENT_PMU;
-		g_strlcpy (hotplug_event->pmu.pmu_path, path, sizeof (hotplug_event->pmu.pmu_path));
-		hotplug_event->pmu.pmu_type = PMU_TYPE_LID_BUTTON;
-		hotplug_event_enqueue (hotplug_event);
+		/* Add lid button */
+		snprintf (path, sizeof (path), "%s/pmu/info", get_hal_proc_path ());
+		pmu_synthesize_item (path, PMU_TYPE_LID_BUTTON);
+
+		/* Add Laptop Panel */
+		snprintf (path, sizeof (path), "%s/pmu/info", get_hal_proc_path ());
+		pmu_synthesize_item (path, PMU_TYPE_LAPTOP_PANEL);
+
+		/* If the machine has got battery bays then this is a laptop. */
+		hal_device_property_set_string (computer, "system.formfactor", "laptop");
 	}
 
 	/* setup timer for things that we need to poll */
@@ -389,10 +455,19 @@ static PMUDevHandler pmudev_handler_lid_button = {
 	.remove      = pmu_generic_remove
 };
 
+static PMUDevHandler pmudev_handler_laptop_panel = { 
+	.pmu_type    = PMU_TYPE_LAPTOP_PANEL,
+	.add         = pmu_generic_add,
+	.compute_udi = pmu_laptop_panel_compute_udi,
+	.refresh     = laptop_panel_refresh,
+	.remove      = pmu_generic_remove
+};
+
 static PMUDevHandler *pmu_handlers[] = {
 	&pmudev_handler_battery,
 	&pmudev_handler_ac_adapter,
 	&pmudev_handler_lid_button,
+	&pmudev_handler_laptop_panel,
 	NULL
 };
 
