@@ -1,12 +1,12 @@
 /***************************************************************************
- * CVSID: $Id: property.c,v 1.13 2005/05/24 17:42:14 david Exp $
+ * CVSID: $Id: property.c,v 1.17 2006/01/21 19:08:27 david Exp $
  *
  * property.c : HalProperty methods
  *
  * Copyright (C) 2003 David Zeuthen, <david@fubar.dk>
  * Copyright (C) 2004 Novell, Inc.
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  **************************************************************************/
 
@@ -229,6 +229,7 @@ hal_property_to_string (HalProperty *prop)
 		char buf[256];
 		
 		i = 0;
+		buf[0] = '\0';
 		for (iter = hal_property_get_strlist (prop); 
 		     iter != NULL && i < sizeof(buf); 
 		     iter = g_slist_next (iter)) {
@@ -471,3 +472,19 @@ hal_property_strlist_remove (HalProperty *prop, const char *value)
 
 	return FALSE;
 }
+
+gboolean 
+hal_property_strlist_clear (HalProperty *prop)
+{
+	GSList *elem;
+
+	g_return_val_if_fail (prop != NULL, FALSE);
+	g_return_val_if_fail (prop->type == HAL_PROPERTY_TYPE_STRLIST, FALSE);
+
+	for (elem = prop->strlist_value; elem != NULL; elem = g_slist_next (elem)) {
+		g_free (elem->data);
+	}
+	g_slist_free (prop->strlist_value);
+
+	return FALSE;
+}
