@@ -1,12 +1,12 @@
 /***************************************************************************
- * CVSID: $Id: property.h,v 1.7 2005/02/14 18:20:04 david Exp $
+ * CVSID: $Id: property.h,v 1.10 2006/01/21 06:36:51 david Exp $
  *
  * property.c : HalProperty methods
  *
  * Copyright (C) 2003 David Zeuthen, <david@fubar.dk>
  * Copyright (C) 2004 Novell, Inc.
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  **************************************************************************/
 
@@ -91,6 +91,7 @@ gboolean      hal_property_strlist_add        (HalProperty  *prop,
 					       const char *value);
 gboolean      hal_property_strlist_remove     (HalProperty  *prop,
 					       const char *value);
+gboolean      hal_property_strlist_clear      (HalProperty  *prop);
 
 
 void          hal_property_set_attribute      (HalProperty *prop,
