@@ -1,11 +1,11 @@
 /***************************************************************************
- * CVSID: $Id: util.c,v 1.18 2005/06/27 19:13:03 david Exp $
+ * CVSID: $Id: util.c,v 1.36 2006/01/23 12:15:50 kay Exp $
  *
  * util.c - Various utilities
  *
  * Copyright (C) 2004 David Zeuthen, <david@fubar.dk>
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  **************************************************************************/
 
@@ -31,8 +31,9 @@
 #include <stdarg.h>
 #include <string.h>
 #include <errno.h>
+#include <time.h>
 #include <ctype.h>
-#include <sys/types.h>
+#include <stdint.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <signal.h>
@@ -49,6 +50,54 @@
 
 #include "hald_dbus.h"
 #include "util.h"
+#include "hald_runner.h"
+
+typedef struct {
+	int last_level;
+	int last_chargeRate;
+	time_t last_time;
+} batteryInfo;
+
+GHashTable *saved_battery_info = NULL;
+
+/** Given all the required parameters, this function will return the percentage
+ *  charge remaining. There are lots of checks here as ACPI is often broken.
+ *
+ *  @param  id                  Optional ID given to this battery. Unused at present.
+ *  @param  chargeLevel         The current charge level of the battery (typically mWh)
+ *  @param  chargeLastFull      The last "full" charge of the battery (typically mWh)
+ *  @return                     Percentage, -1 if invalid
+ */
+int 
+util_compute_percentage_charge (const char *id,
+			     int chargeLevel,
+			     int chargeLastFull)
+{
+	int percentage;
+	/* make sure we have chargelevel */
+	if (chargeLevel <= 0) {
+		HAL_WARNING (("chargeLevel %i, returning -1!", chargeLevel));
+		return -1;
+	}
+	/* make sure not division by zero */
+	if (chargeLastFull > 0)
+		percentage = ((double) chargeLevel / (double) chargeLastFull) * 100;
+	else {
+		HAL_WARNING (("chargeLastFull %i, percentage returning -1!", chargeLastFull));
+		return -1;
+	}
+	/* Some bios's will report this higher than 100, limit it here */
+	if (percentage > 100) {
+		HAL_WARNING (("Percentage %i, returning 100!", percentage));
+		return 100;
+	}
+	/* Something really isn't right if we get a negative... */
+	if (percentage < 0) {
+		HAL_WARNING (("Percentage %i, returning -1!", percentage));
+		return -1;
+	}
+	return percentage;
+}
 
 /** Given all the required parameters, this function will return the number 
  *  of seconds until the battery is charged (if charging) or the number
@@ -60,7 +109,8 @@
  *  @param  chargeLastFull      The last "full" charge of the battery (typically mWh)
  *  @param  isDischarging       If battery is discharging
  *  @param  isCharging          If battery is charging
- *  @return                     Number of seconds, or zero if n/a
+ *  @param  guessChargeRate     If ignore chargeRate and guess them.
+ *  @return                     Number of seconds, or -1 if invalid
  */
 int 
 util_compute_time_remaining (const char *id,
@@ -68,16 +118,94 @@ util_compute_time_remaining (const char *id,
 			     int chargeLevel,
 			     int chargeLastFull,
 			     gboolean isDischarging,
-			     gboolean isCharging)
+			     gboolean isCharging,
+			     gboolean guessChargeRate)
 {
+	int remaining_time = 0;
 
-	if (chargeRate <= 0)
-		return 0;
-	if (isDischarging)
-		return ((double) chargeLevel / (double) chargeRate) * 60 * 60;
-	if (isCharging)
-		return ((double) (chargeLastFull - chargeLevel) / (double) chargeRate) * 60 * 60;
-	return 0;
+	/* should not get negative values */
+	if (chargeRate < 0 || chargeLevel < 0 || chargeLastFull < 0) {
+		HAL_WARNING (("chargeRate, chargeLevel or chargeLastFull < 0, returning -1"));
+		return -1;
+	}
+	/* batteries cannot charge and discharge at the same time */
+	if (isDischarging && isCharging) {
+		HAL_WARNING (("isDischarging & isCharging TRUE, returning -1"));
+		return -1;
+	}
+	/* 
+	 * Some laptops don't supply any rate info, but that's no reason for HAL not
+	 * to. We use the current and previous chargeLevel to estimate the rate.
+	 * The info is stored in a GHashTable because there could be more than one battery.
+	 */
+	if (chargeRate == 0 || guessChargeRate) {
+		batteryInfo *battery_info;
+		time_t cur_time = time(NULL);
+
+		/* Initialize the save_battery_info GHashTable */
+		if (!saved_battery_info) 
+			saved_battery_info = g_hash_table_new(g_str_hash, g_str_equal);
+
+		if ((battery_info = g_hash_table_lookup(saved_battery_info, id))) {
+			/* check this to prevent division by zero */
+			if ((cur_time == battery_info->last_time) || (chargeLevel == battery_info->last_level)) {
+				/* if we can't calculate because nothing changed, use last 
+				 * chargeRate to prevent removing battery.remaining_time.
+				 */
+				chargeRate = battery_info->last_chargeRate;
+			} else {
+				chargeRate = ((chargeLevel - battery_info->last_level) * 60 * 60) / (cur_time - battery_info->last_time);
+				/*
+				 * During discharging chargeRate would be negative, which would
+				 * mess up the the calculation below, so we make sure it's always
+				 * positive.
+				 */ 
+				chargeRate = (chargeRate > 0) ? chargeRate : -chargeRate;
+	
+				battery_info->last_level = chargeLevel;
+				battery_info->last_time = cur_time;
+				battery_info->last_chargeRate = chargeRate;
+			}
+		} else {
+			battery_info = g_new0(batteryInfo, 1);
+			g_hash_table_insert(saved_battery_info, (char*) id, battery_info);
+
+			battery_info->last_level = chargeLevel;
+			battery_info->last_time = cur_time;
+			battery_info->last_chargeRate = 0;
+ 			return -1;
+		}
+	} 
+
+	if (chargeRate == 0)
+		return -1;
+
+	if (isDischarging) { 
+		remaining_time = ((double) chargeLevel / (double) chargeRate) * 60 * 60;
+	} else if (isCharging) {
+		/* 
+		 * Some ACPI BIOS's don't update chargeLastFull, 
+		 * so return 0 as we don't know how much more there is left
+		 */
+		if (chargeLevel > chargeLastFull ) {
+			HAL_WARNING (("chargeLevel > chargeLastFull, returning -1"));
+			return -1;
+		}
+		remaining_time = ((double) (chargeLastFull - chargeLevel) / (double) chargeRate) * 60 * 60;
+	}
+	
+	/* This shouldn't happen, but check for completeness */
+	if (remaining_time < 0) {
+		HAL_WARNING (("remaining_time %i, returning -1", remaining_time));
+		remaining_time = -1;
+	}
+	/* Battery life cannot be above 60 hours */
+	else if (remaining_time > 60*60*60) {
+		HAL_WARNING (("remaining_time *very* high, returning -1"));
+		remaining_time = -1;
+	}
+
+	return remaining_time;
 }
 
 gboolean 
@@ -407,217 +535,6 @@ out:
 	;
 }
 
-void
-hal_util_terminate_helper (HalHelperData *ed)
-{
-	if (ed->already_issued_kill) {
-		HAL_INFO (("Already issued SIGTERM for pid %d, udi %s",
-			   ed->pid, ed->d != NULL ? ed->d->udi : "(finalized object)"));
-		goto out;
-	}
-
-	HAL_INFO (("killing %d for udi %s", ed->pid, ed->d != NULL ? ed->d->udi : "(finalized object)"));
-
-	/* kill kenny! kill it!! */
-	ed->already_issued_kill = TRUE;
-	kill (ed->pid, SIGTERM);
-
-	if (ed->timeout_watch_id != (guint) -1) {
-		g_source_remove (ed->timeout_watch_id);
-		ed->timeout_watch_id = -1;
-	}
-
-	if (!ed->already_issued_callback) {
-		ed->already_issued_callback = TRUE;
-		ed->cb (ed->d, TRUE, -1, ed->data1, ed->data2, ed);
-	}
-
-	/* ed will be cleaned up when helper_child_exited reaps the child */
-out:
-	;
-}
-
-static gboolean
-helper_child_timeout (gpointer data)
-{
-	HalHelperData *ed = (HalHelperData *) data;
-
-	HAL_INFO (("child timeout for pid %d", ed->pid));
-
-	/* kill kenny! kill it!! */
-	ed->already_issued_kill = TRUE;
-	kill (ed->pid, SIGTERM);
-
-	ed->timeout_watch_id = -1;
-
-	if (!ed->already_issued_callback) {
-		ed->already_issued_callback = TRUE;
-		ed->cb (ed->d, TRUE, -1, ed->data1, ed->data2, ed);
-	}
-
-	/* ed will be cleaned up when helper_child_exited reaps the child */
-	return FALSE;
-}
-
-static GSList *running_helpers = NULL;
-
-static void 
-helper_device_object_finalized (gpointer data, GObject *where_the_object_was)
-{
-	HalHelperData *ed = (HalHelperData *) data;
-
-	HAL_INFO (("device object finalized for helper with pid %d", ed->pid));
-
-	ed->d = NULL;
-	hal_util_terminate_helper (ed);
-}
-
-static void 
-helper_child_exited (GPid pid, gint status, gpointer data)
-{
-	HalHelperData *ed = (HalHelperData *) data;
-
-	HAL_INFO (("child exited for pid %d", pid));
-
-	if (ed->timeout_watch_id != (guint) -1)
-		g_source_remove (ed->timeout_watch_id);
-	g_spawn_close_pid (ed->pid);
-
-	if (ed->d != NULL)
-		g_object_weak_unref (G_OBJECT (ed->d), helper_device_object_finalized, ed);
-
-	if (!ed->already_issued_callback)
-		ed->cb (ed->d, FALSE, WEXITSTATUS (status), ed->data1, ed->data2, ed);
-
-	running_helpers = g_slist_remove (running_helpers, ed);
-
-	g_free (ed);
-}
-
-static gboolean
-helper_add_property_to_env (HalDevice *device, HalProperty *property, gpointer user_data)
-{
-	char *prop_upper, *value;
-	char *c;
-	gchar ***ienvp = (gchar ***) user_data;
-	gchar **envp;
-
-	envp = *ienvp;
-	*ienvp = *ienvp + 1;
-
-	prop_upper = g_ascii_strup (hal_property_get_key (property), -1);
-	
-	/* periods aren't valid in the environment, so replace them with
-	 * underscores. */
-	for (c = prop_upper; *c; c++) {
-		if (*c == '.')
-			*c = '_';
-	}
-	
-	value = hal_property_to_string (property);
-	
-	*envp = g_strdup_printf ("HAL_PROP_%s=%s", prop_upper, value);
-
-	g_free (value);
-	g_free (prop_upper);
-
-	return TRUE;
-}
-
-HalHelperData *
-hal_util_helper_invoke (const gchar *command_line, gchar **extra_env, HalDevice *d, 
-			gpointer data1, gpointer data2, HalHelperTerminatedCB cb, guint timeout)
-{
-	HalHelperData *ed;
-	gint argc;
-	gchar **argv;
-	gchar **envp;
-	gchar **ienvp;
-	GError *err = NULL;
-	guint num_env_vars;
-	guint i, j;
-	guint num_properties;
-	guint num_extras;
-	char *local_addr;
-
-	ed = g_new0 (HalHelperData, 1);
-	ed->data1 = data1;
-	ed->data2 = data2;
-	ed->d = d;
-	ed->cb = cb;
-	ed->already_issued_callback = FALSE;
-	ed->already_issued_kill = FALSE;
-
-	num_properties = hal_device_num_properties (d);
-	if (extra_env == NULL)
-		num_extras = 0;
-	else
-		num_extras = g_strv_length ((gchar **) extra_env);
-	num_env_vars = num_properties + 2 + num_extras;
-	if (hald_is_verbose)
-		num_env_vars++;
-	if (hald_is_initialising)
-		num_env_vars++;
-	if ((local_addr = hald_dbus_local_server_addr ()) != NULL)
-		num_env_vars++;
-
-	envp = g_new (char *, num_env_vars);
-	ienvp = envp;
-	hal_device_property_foreach (d, helper_add_property_to_env, &ienvp);
-	i = num_properties;
-	envp[i++] = g_strdup_printf ("UDI=%s", hal_device_get_udi (d));
-	if (hald_is_verbose)
-		envp[i++] = g_strdup ("HALD_VERBOSE=1");
-	if (hald_is_initialising)
-		envp[i++] = g_strdup ("HALD_STARTUP=1");
-	if (local_addr != NULL)
-		envp[i++] = g_strdup_printf ("HALD_DIRECT_ADDR=%s", local_addr);
-	for (j = 0; j < num_extras; j++) {
-		envp[i++] = g_strdup (extra_env[j]);
-	}
-	envp[i++] = NULL;
-
-	err = NULL;
-	if (!g_shell_parse_argv (command_line, &argc, &argv, &err)) {
-		HAL_ERROR (("Error parsing commandline '%s': %s", command_line, err->message));
-		g_error_free (err);
-		g_free (ed);
-		ed = NULL;
-	} else {
-		err = NULL;
-		if (!g_spawn_async (NULL, 
-				    argv, 
-				    envp, 
-				    G_SPAWN_DO_NOT_REAP_CHILD|G_SPAWN_SEARCH_PATH,
-				    NULL,
-				    NULL,
-				    &ed->pid,
-				    &err)) {
-			HAL_ERROR (("Couldn't spawn '%s' err=%s!", command_line, err->message));
-			g_error_free (err);
-			g_free (ed);
-			ed = NULL;
-		} else {
-			ed->child_watch_id = g_child_watch_add (ed->pid, helper_child_exited, (gpointer) ed);
-			if (timeout > 0)
-				ed->timeout_watch_id = g_timeout_add (timeout, helper_child_timeout, (gpointer) ed);
-			else
-				ed->timeout_watch_id = (guint) -1;
-
-			running_helpers = g_slist_prepend (running_helpers, ed);
-			/* device object may disappear from underneath us - this is
-			 * used to terminate the helper and pass d=NULL in the
-			 * HalHelperTerminatedCB callback
-			 */
-			g_object_weak_ref (G_OBJECT (d), helper_device_object_finalized, ed);
-		}
-	}
-
-	g_strfreev (envp);
-	g_free (argv);
-
-	return ed;
-}
 
 gboolean
 hal_util_path_ascend (gchar *path)
@@ -839,7 +756,7 @@ out:
  *
  *    "design voltage:          10800 mV"
  *
- *  then hal_util_set_int_elem_from_file (d, "battery.bar",
+ *  then hal_util_set_int_elem_from_file (d, "battery.foo",
  *  "/proc/acpi/battery/BAT0", "info", "design voltage", 0) will assign
  *  the integer 10800 to the property "battery.foo" on d.
  *
@@ -983,8 +900,9 @@ typedef struct {
 static void callout_do_next (Callout *c);
 
 static void 
-callout_terminated (HalDevice *d, gboolean timed_out, gint return_code, 
-		    gpointer data1, gpointer data2, HalHelperData *helper_data)
+callout_terminated (HalDevice *d, guint32 exit_type, 
+                   gint return_code, gchar **error, 
+                   gpointer data1, gpointer data2)
 {
 	Callout *c;
 
@@ -1015,8 +933,9 @@ callout_do_next (Callout *c)
 		callback (d, userdata1, userdata2);
 
 	} else {
-		hal_util_helper_invoke (c->programs[c->next_program], c->extra_env, c->d, 
-					(gpointer) c, NULL, callout_terminated, HAL_HELPER_TIMEOUT);
+    hald_runner_run(c->d, c->programs[c->next_program], c->extra_env,
+                    HAL_HELPER_TIMEOUT, callout_terminated,
+                    (gpointer)c, NULL);
 		c->next_program++;
 	}
 }
@@ -1093,27 +1012,21 @@ out:
 	;
 }
 
-/** Kill all helpers we have running; useful when exiting hald.
- *
- *  @return                     Number of childs killed
- */
-unsigned int 
-hal_util_kill_all_helpers (void)
+gchar *
+hal_util_strdup_valid_utf8 (const char *str)
 {
-	unsigned int n;
-	GSList *i;
+	char *endchar;
+	char *newstr;
 
-	n = 0;
-	for (i = running_helpers; i != NULL; i = i->next) {
-		HalHelperData *ed;
+	if (str == NULL)
+		return NULL;
 
-		ed = i->data;
-		HAL_INFO (("Killing helper with pid %d", ed->pid));
-		kill (ed->pid, SIGTERM);
-		n++;
+	newstr = g_strdup (str);
+
+	while (!g_utf8_validate (str, -1, (const char **) &endchar)) {
+		*endchar = '?';
 	}
-
-	return n;
+	return newstr;
 }
 
 void
