@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: libhal-storage.h,v 1.10 2005/06/28 15:47:51 david Exp $
+ * CVSID: $Id: libhal-storage.h,v 1.17 2006/02/23 00:23:13 david Exp $
  *
  * libhal-storage.h : HAL convenience library for storage devices and volumes
  *
@@ -7,7 +7,7 @@
  *
  * Author: David Zeuthen <davidz@redhat.com>
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  **************************************************************************/
 
@@ -121,7 +121,14 @@ typedef enum {
 	LIBHAL_STORAGE_ICON_DISC_DVDR                      = 0x30005,
 	LIBHAL_STORAGE_ICON_DISC_DVDRW                     = 0x30006,
 	LIBHAL_STORAGE_ICON_DISC_DVDPLUSR                  = 0x30007,
-	LIBHAL_STORAGE_ICON_DISC_DVDPLUSRW                 = 0x30008
+	LIBHAL_STORAGE_ICON_DISC_DVDPLUSRW                 = 0x30008,
+	LIBHAL_STORAGE_ICON_DISC_DVDPLUSRWDL               = 0x30009,
+	LIBHAL_STORAGE_ICON_DISC_BDROM                     = 0x3000a,
+	LIBHAL_STORAGE_ICON_DISC_BDR                       = 0x3000b,
+	LIBHAL_STORAGE_ICON_DISC_BDRE                      = 0x3000c,
+	LIBHAL_STORAGE_ICON_DISC_HDDVDROM                  = 0x3000d,
+	LIBHAL_STORAGE_ICON_DISC_HDDVDR                    = 0x3000e,
+	LIBHAL_STORAGE_ICON_DISC_HDDVDRW                   = 0x3000f
 } LibHalStoragePolicyIcon;
 
 typedef struct {
@@ -176,7 +183,14 @@ typedef enum {
 	LIBHAL_DRIVE_CDROM_CAPS_DVDR       = 0x0020,
 	LIBHAL_DRIVE_CDROM_CAPS_DVDRW      = 0x0040,
 	LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSR   = 0x0080,
-	LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRW  = 0x0100
+	LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRW  = 0x0100,
+	LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRDL = 0x0200,
+	LIBHAL_DRIVE_CDROM_CAPS_BDROM      = 0x0400,
+	LIBHAL_DRIVE_CDROM_CAPS_BDR        = 0x0800,
+	LIBHAL_DRIVE_CDROM_CAPS_BDRE       = 0x1000,
+	LIBHAL_DRIVE_CDROM_CAPS_HDDVDROM   = 0x2000,
+	LIBHAL_DRIVE_CDROM_CAPS_HDDVDR     = 0x4000,
+	LIBHAL_DRIVE_CDROM_CAPS_HDDVDRW    = 0x8000
 } LibHalDriveCdromCaps;
 
 LibHalDrive         *libhal_drive_from_udi                    (LibHalContext *hal_ctx, 
@@ -236,7 +250,9 @@ char        *libhal_drive_policy_default_get_managed_keyword_secondary (LibHalCo
 typedef enum {
 	LIBHAL_VOLUME_USAGE_MOUNTABLE_FILESYSTEM,
 	LIBHAL_VOLUME_USAGE_PARTITION_TABLE,
-	LIBHAL_VOLUME_USAGE_RAID_MEMBER
+	LIBHAL_VOLUME_USAGE_RAID_MEMBER,
+	LIBHAL_VOLUME_USAGE_CRYPTO,
+	LIBHAL_VOLUME_USAGE_UNKNOWN
 } LibHalVolumeUsage;
 
 typedef enum {
@@ -249,7 +265,13 @@ typedef enum {
 	LIBHAL_VOLUME_DISC_TYPE_DVDRW       = 0x06,
 	LIBHAL_VOLUME_DISC_TYPE_DVDPLUSR    = 0x07,
 	LIBHAL_VOLUME_DISC_TYPE_DVDPLUSRW   = 0x08,
-	LIBHAL_VOLUME_DISC_TYPE_DVDPLUSR_DL = 0x09
+	LIBHAL_VOLUME_DISC_TYPE_DVDPLUSR_DL = 0x09,
+	LIBHAL_VOLUME_DISC_TYPE_BDROM       = 0x0a,
+	LIBHAL_VOLUME_DISC_TYPE_BDR         = 0x0b,
+	LIBHAL_VOLUME_DISC_TYPE_BDRE        = 0x0c,
+	LIBHAL_VOLUME_DISC_TYPE_HDDVDROM    = 0x0d,
+	LIBHAL_VOLUME_DISC_TYPE_HDDVDR      = 0x0e,
+	LIBHAL_VOLUME_DISC_TYPE_HDDVDRW     = 0x0f
 } LibHalVolumeDiscType;
 
 LibHalVolume     *libhal_volume_from_udi                      (LibHalContext *hal_ctx, 
@@ -275,6 +297,10 @@ const char          *libhal_volume_get_mount_point               (LibHalVolume
 const char          *libhal_volume_get_uuid                      (LibHalVolume     *volume);
 const char          *libhal_volume_get_storage_device_udi        (LibHalVolume     *volume);
 
+const char          *libhal_volume_crypto_get_backing_volume_udi (LibHalVolume     *volume);
+char                *libhal_volume_crypto_get_clear_volume_udi   (LibHalContext    *hal_ctx, LibHalVolume *volume);
+
+
 dbus_bool_t          libhal_volume_disc_has_audio             (LibHalVolume     *volume);
 dbus_bool_t          libhal_volume_disc_has_data              (LibHalVolume     *volume);
 dbus_bool_t          libhal_volume_disc_is_blank              (LibHalVolume     *volume);
@@ -284,6 +310,8 @@ LibHalVolumeDiscType libhal_volume_get_disc_type              (LibHalVolume
 
 int               libhal_volume_get_msdos_part_table_type     (LibHalVolume     *volume);
 
+dbus_bool_t       libhal_volume_should_ignore 	              (LibHalVolume     *volume);
+
 char             *libhal_volume_policy_compute_size_as_string (LibHalVolume     *volume);
 
 char             *libhal_volume_policy_compute_display_name   (LibHalDrive         *drive, 
