@@ -3,19 +3,9 @@
  *
  * Copyright (C) 2004 Kay Sievers <kay.sievers@vrfy.org>
  *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation; either
- *	version 2.1 of the License, or (at your option) any later version.
- *
- *	This library is distributed in the hope that it will be useful,
- *	but WITHOUT ANY WARRANTY; without even the implied warranty of
- *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- *	Lesser General Public License for more details.
- *
- *	You should have received a copy of the GNU Lesser General Public
- *	License along with this library; if not, write to the Free Software
- *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ *	This program is free software; you can redistribute it and/or modify it
+ *	under the terms of the GNU General Public License as published by the
+ *	Free Software Foundation version 2 of the License.
  */
 
 #ifndef _GNU_SOURCE
@@ -36,24 +26,23 @@
 #include "volume_id.h"
 #include "logging.h"
 #include "util.h"
-#include "jfs.h"
 
 struct jfs_super_block {
-	__u8	magic[4];
-	__u32	version;
-	__u64	size;
-	__u32	bsize;
-	__u32	dummy1;
-	__u32	pbsize;
-	__u32	dummy2[27];
-	__u8	uuid[16];
-	__u8	label[16];
-	__u8	loguuid[16];
-} __attribute__((__packed__));
+	uint8_t		magic[4];
+	uint32_t	version;
+	uint64_t	size;
+	uint32_t	bsize;
+	uint32_t	dummy1;
+	uint32_t	pbsize;
+	uint32_t	dummy2[27];
+	uint8_t		uuid[16];
+	uint8_t		label[16];
+	uint8_t		loguuid[16];
+} PACKED;
 
 #define JFS_SUPERBLOCK_OFFSET			0x8000
 
-int volume_id_probe_jfs(struct volume_id *id, __u64 off)
+int volume_id_probe_jfs(struct volume_id *id, uint64_t off)
 {
 	struct jfs_super_block *js;
 
